/* convert a structure factor file from one format to another */

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <ctype.h>
#include <unistd.h>
#include <sys/types.h>
#include "sf_convert.h"
#include "util.h"
#include "TableFile.h"
#include "CifFile.h"
#include "CifParserBase.h"

extern char  PDBID[80], AUDIT[512];
extern int CIF_CHECK;

void cif_dic_check(char *ciffile);


void reformat_sfhead(char *file, char *outfile)
{
    char infile[256],command[256], pdbid[20], str[512], block_id[80];
    char  **line=NULL, **items=NULL, **item1=NULL,tmpf[256],tmpf_new[256];
    int i, j, nline, nh=0, nb=0, nf=0, k, m, nnf;
    FILE *fw, *fout, *ftmp;


    strcpy(pdbid, ".");
    if ((fw=fopen(file, "r"))==NULL){ //try pdbid
        sprintf(command, "auto -sf %s", file);
        system(command);
        strcpy(pdbid,file);
        sprintf(infile, "r%ssf.ent", pdbid);
        sprintf(outfile, "r%ssf.ent_new", pdbid);
        
    }else{
        strcpy(infile, file);
    }
    if (fw) fclose(fw);
    
    fout=fopen(outfile, "w");
    
    line=get_lines_from_file(infile, &nline);
    
    nb=0;
    for(i=0; i<nline; i++){
        if (strchr(line[i],'_')){rid_of_front_space(line[i]);}
        
                   
        if (!strncmp(line[i], "data_", 5)){
            strcpy(block_id,line[i]+5);
            rid_of_front_end_space(block_id);
            check_data_block_name(block_id, nb);
            nb++;

            if(nb==1){ //get pdbid
                get_string(line[i],6, 9, pdbid);
                rid_of_front_end_space(pdbid);
            }
                
            items=set_memory_2d(nline, 400);
                
            sprintf(tmpf, "data_block_%d", nb);
            ftmp=fopen(tmpf, "w");
            nf=0;
            for (j=i; j<nline; j++){
                if (strchr(line[j],'_') || strchr(line[j],'#')){rid_of_front_space(line[j]);}
                if ( j-i>10 &&  !strncmp(line[j], "data_", 5) ){
                    i=j-1;
                    break;
                }
                strcpy(items[nf], line[j]);
                nf++;
                
                fprintf(ftmp,"%s", line[j]);
            }
            fclose(ftmp);
            if (nf<10) {
                printf("Error: No data in this block %d (%s)\n", nb, file);
                continue;
            }
            
            delete_file(tmpf);            

            nnf=0; //number of _refln. in the block
            sprintf(tmpf, "head_part_%d", nb);
            fw=fopen(tmpf,"w");
                
            for(k=0; k<nf; k++){
                if (k<nf-1 && !strncmp(items[k], "loop_", 5) && 
                    (!strncmp(items[k+1], "_refln.", 7) ||!strncmp(items[k+1], "_phasing_MIR_der_refln.", 23)) ){
                    item1=set_memory_2d(nf, 400);
                    for (m=k; m<nf; m++){
                        if (m-k>100 && (items[m][0]=='_' || !strncmp(items[m], "loop_", 5) 
                                        )){
                            break;
                        }
                        strcpy(item1[nnf], items[m]);
                        nnf++;
                    }
                    k=m-1;
                }else{
                    fprintf(fw, "%s", items[k]);
                }
            }
            fclose(fw);
            sprintf(tmpf_new, "%s_head_%d",file, nb);
            update_items(tmpf, tmpf_new, pdbid, nb);
            cifparse(tmpf_new, tmpf_new, 100) ; 
            
            if(CIF_CHECK==1) cif_dic_check(tmpf_new);
            
            ftmp=fopen(tmpf_new, "r");
            nh=0;
            while(fgets(str, sizeof str, ftmp)!=NULL){
                fprintf(fout, "%s", str);
                nh++;
            }
            fclose(ftmp);
            
            for (k=0; k<nnf; k++){
                fprintf(fout, "%s", item1[k]);
            }
            fprintf(fout, "\n");
            
            if (nf) free_memory_2d(items, nf);
            if (nnf) free_memory_2d(item1, nnf);
            delete_file(tmpf);
            delete_file(tmpf_new);
            
            printf("%s_new: block=%d nall= %7d nref= %7d %7d diff= %d nh= %d\n",
                   infile, nb, nline, nf, nnf, nf-nnf, nh);
        }
            
            
        
    }
    fclose(fout);
    
}
void cif_dic_check(char *ciffile)
{
    char command[256];
    printf("Checking pdbx mmCIF dictionary by (cif_check)..\n");
    
    sprintf(command,  "~annotator/bin/cif_check.csh  %s >/dev/null", ciffile);
    system(command);
}



void update_items(char *file,char *newfile, char *pdbid, int nblock)
{
    char  tmp[256], tmp1[256],date[30], correct[512], add[256],change[256], removed[256];
    char **line, **line1, tmpout[512], comment[9000];
    int i,j,k, nline, nline1, n1=0,n2=0,n3=0, ncell=0, nreflns=0,ndiffr=0,nwave=0;
    int nloop=0, entry=0, nexptl=0;
    FILE *fw=NULL, *fw1=NULL;


    fw=fopen(newfile, "w");

    strcpy(date, "");
    strcpy(correct, "");
    strcpy(add, "");
    strcpy(change, "");
    strcpy(removed, "");
    strcpy(comment, "");

    line1=get_lines_from_file(file, &nline1);

    sprintf(tmpout, "%s__tmp", file);
    fw1=fopen(tmpout, "w");
    
    for(k=0; k<nline1 ; k++){
        if(!strncmp(line1[k], "_cell.entry_id", 14)){
            ncell++;
        }else if(!strncmp(line1[k], "_reflns.entry_id", 16)) {
            nreflns++;
        }else if(!strncmp(line1[k], "_diffrn.details", 15)) {
            ndiffr++;
        }else if(!strncmp(line1[k], "_diffrn_radiation_wavelength.wavelength", 39)){
            nwave++;
        }else if(!strncmp(line1[k], "_entry.id", 9)){
            entry++;
        }else if(!strncmp(line1[k], "_exptl_crystal.id", 17)){
            nexptl++;
        }else if (line1[k][0]=='#') {
            guess_comment(line1[k], comment);
        }else if(k<nline1-1 && !strncmp(line1[k],"loop_", 5) && !strncmp(line1[k+1],"_reflns.", 8) ){
            nloop=1;
        }
        
           
        if(!strncmp(line1[k],"loop_", 5) &&
           strstr(line1[k+1],"_symmetry_equiv.") &&
           strstr(line1[k+2],"_symmetry_equiv.") &&
           !strstr(line1[k+3],"_symmetry_equiv.")&&
           (strchr(line1[k+3],'_') ||strchr(line1[k+3],'#'))){
            k=k+2;
            printf("Warning: (%s) empty table _symmetry_equiv (removed)\n", pdbid);
            strcat (removed, "_symmetry_equiv, ");
            continue;
        }else{
            if (!strncmp(line1[k], "_reflns.CCP4_wavelength_id", 26)){
                printf("Warning: (%s) Possible looped  _reflns \n", pdbid);
                
            }
            
            if (!strncmp(line1[k], "_reflns.entry_id",16)){
                if (sscanf(line1[k], "%s %s" , tmp1,tmp)==2){
                    continue;
                }
            }
            
            fprintf(fw1, "%s", line1[k]);
        }
        
    }
    fclose(fw1);
    if (ncell>1) printf("Warning: (%s) _cell.entry_id appeared twice(manual check).\n", pdbid);
    if (nreflns>1) printf("Warning: (%s) _reflns.entry_id appeared twice(manual check).\n", pdbid);
    if (strlen(comment)>10){
        printf("Warning: (%s) Text below is put into _diffrn.details. Check carefully!\n", pdbid);
        printf("Text in %s: (%s)\n", pdbid, comment);
        strcat(add, "_diffrn to hold comments, ");
    }
    
    
    cifparse(tmpout, tmpout, 200) ;
    line=get_lines_from_file(tmpout, &nline);
    
    for(i=0; i<nline; i++){ //remove items (then add _audit &_diffrn back)
        if (i<nline-1 && (!strncmp(line[i],"loop_", 5) || !strncmp(line[i],"#", 1)) &&
            (!strncmp(line[i+1],"_database_2.", 12) ||
             !strncmp(line[i+1],"_atom_sites.", 12) ||
             !strncmp(line[i+1],"_audit.", 7) ||
             !strncmp(line[i+1],"_diffrn.", 8)
            )){

            if (!strncmp(line[i+1],"_database_2.", 12)){
                strcat (removed, "_database_2, ");
            }else if (!strncmp(line[i+1],"_atom_sites.", 12)){
                strcat (removed, "_atom_sites, ");
            }
            
            for (j=i; j<nline; j++){
                if (j-i>0 && (line[j][0]=='#'|| strstr(line[j], "loop_")) ){
                    i=j-1;
                    break;
                }
            }

        }else if(!strncmp(line[i],"_diffrn_radiation_wavelength.id", 31) && nwave==0 ){
            strcat(removed, "_diffrn_radiation_wavelength.id, ");
            continue;
            
        }else if(!strncmp(line[i],"_diffrn_radiation.id", 20) ){
            replace_first_str(line[i], "_diffrn_radiation.", "_diffrn_radiation_wavelength.", tmp);
            fprintf(fw, "%s",  tmp);
            
        }else if(!strncmp(line[i],"_diffrn_radiation.pdbx_wavelength", 33) ){
            replace_first_str(line[i], "_diffrn_radiation.pdbx_wavelength",
                              "_diffrn_radiation_wavelength.wavelength", tmp);
            fprintf(fw, "%s",  tmp);
            strcat(change, "_diffrn_radiation to _diffrn_radiation_wavelength, ");
            
        }else if(!strncmp(line[i],"_exptl.crystal_F_000", 20)){
            
            strcat(change, "_exptl.crystal_F_000 to _exptl_crystal.F_000, ");
            replace_first_str(line[i], "_exptl.crystal_F_000", "_exptl_crystal.F_000", tmp);
//            fprintf(fw, "_exptl_crystal.id   1");
//            strcat(add, "_exptl_crystal.id, ");
            fprintf(fw, "%s",  tmp);
            continue;
            
            
        }else if(!strncmp(line[i],"_cell.", 6)){
            n1++;
            if (sscanf(line[i], "%s %s", tmp,  tmp1)==2){
                if (strstr(tmp,"_cell.formula_units_Z") &&  atoi(tmp1)==0 ) {
                    strcat(removed, "_cell.formula_units_Z, ");
                    continue;
                }
            }
            
                
            if (n1==1 && !strstr_case(line[i],"entry_id")){
                fprintf(fw, "_cell.entry_id    %s \n", pdbid);
                fprintf(fw, "%s", line[i]);
                strcat(add, "_cell.entry_id, ");
            }else{
                fprintf(fw, "%s", line[i]);
            }
            
        }else if(!strncmp(line[i],"_symmetry.", 10)){
            n2++;
            if (n2==1 && !strstr_case(line[i],"entry_id")){
                fprintf(fw, "_symmetry.entry_id    %s \n", pdbid);
                fprintf(fw, "%s", line[i]);
                strcat(add, "_symmetry.entry_id, ");
            }else{
                fprintf(fw, "%s", line[i]);
            }
            
        }else if(!strncmp(line[i],"_audit_author.ordinal", 21)){
            replace_first_str(line[i], ".ordinal", ".pdbx_ordinal", tmp);
            fprintf(fw, "%s", tmp);
            strcat(change, "_audit_author.ordinal to _audit_author.pdbx_ordinal, ");
            
        }else if(strstr_case(line[i],"_refln.scale_group_code   1") &&
                 !strncmp(line[i],"_refln.", 7) &&
                 i<nline-1 && strncmp(line[i+1],"_refln.", 7)){
            strcat(removed, "(_refln.scale_group_code  1), ");
            continue; 
            
        }else if(!strncmp(line[i],"_reflns.", 8)){
            n3++;
            if (n3==1) {
                strcat(change, "_reflns to _diffrn_reflns, ");
                if (nloop==1){
                    fprintf(fw, "_diffrn_reflns.diffrn_id   \n");
                }else{
                    fprintf(fw, "_diffrn_reflns.diffrn_id   1 \n");
                }
                    
                strcat(add, "_diffrn_reflns.diffrn_id, ");
            }
            if (!strncmp(line[i],"_reflns.entry_id", 16))continue;
            
            strcpy(tmp,strchr(line[i],'.')+1);
            
            if(strstr(tmp, "d_resolution_high")){
                replace_first_str(tmp, "d_resolution_high", "pdbx_d_res_high", tmp);
            }else if(strstr(tmp, "d_resolution_low")){
                replace_first_str(tmp, "d_resolution_low", "pdbx_d_res_low", tmp);
            }else if (strstr(tmp, "number_all")){
                replace_first_str(tmp, "number_all", "number", tmp);
            }else if (strstr(tmp, "number_obs")){
                replace_first_str(tmp, "number_obs", "pdbx_number_obs", tmp);
            }
            

            
            fprintf(fw, "_diffrn_reflns.%s", tmp);
            
        }else{
            fprintf(fw, "%s", line[i]);
            
        }
    }
    if (ndiffr==0 && n3>0 && !strlen(comment)){
        fprintf(fw, "#\n_diffrn.crystal_id   1 \n");
        fprintf(fw, "_diffrn.id   1 \n#\n");
        strcat(add, "_diffrn.id, _diffrn.crystal_id, ");
    }

    if (entry==0){
        fprintf(fw, "#\n_entry.id   %s \n", pdbid);
        strcat(add, "entry.id, ");
    }
    
    if (nexptl==0){
        fprintf(fw, "#\n_exptl_crystal.id   1 \n");
        strcat(add, "_exptl_crystal.id, ");
    }
    
    
    if (strlen(add)>1) {
        strcat (correct, "ADDED: ");
        strcat (correct, add);
        strcat (correct, "\n");
    }
    
    if (strlen(removed)>1){
        strcat (correct, "REMOVED: ");
        strcat (correct, removed);
        strcat (correct, "\n");
    }
    
    if (strlen(change)>1) {
        strcat (correct, "CHANGED: ");
        strcat (correct, change);
        strcat (correct, "\n");
    }
    
    if (strlen(correct)<5) strcpy(correct, " reformated header part ");
    
    parse_and_update(fw, tmpout,  date,  correct, comment, nblock);
    
    delete_file(tmpout);
    
    fclose(fw);
    
    
}

void guess_comment(char *line, char *comment)
{
    char str[512];
    int c, i, len=0, nlett=0;

    strcpy(str, line);
    
    len=strlen(str);

    for (i=0; i<len; i++){
        c=str[i];
        
        if (c>=97 && c<=122){ nlett++;}
    }
    if (nlett>10){ //possible comments
        replace_char(str,'#', ' ');
        replace_char(str,'\n', ':');
        strcat (comment,str);
    }
    rid_of_front_end_space(comment);

}

void get_cifobj(char *infile, CifFile *fobj,const char *Bid)
{
    string diags, blockId;
    
    CifFile * fobjR = NULL;
    CifParser* cifParserR = NULL;
    fobjR = new CifFile();
    cifParserR = new CifParser(fobjR, fobjR->GetVerbose());
    cifParserR->Parse(infile, diags);
    delete(cifParserR);
    

    blockId = fobjR->GetFirstBlockName();
    Bid=blockId.c_str();
}


void parse_and_update(FILE *fout, char *infile,  char *date,  char *correct,
                      char *comment, int nblock)
{
    
    char **aud_date=NULL,**aud_meth=NULL,**aud_rec=NULL, **aud_id=NULL;
    char **dif_id=NULL,**dif_cid=NULL,**dif_detail=NULL, **temp=NULL, **treat=NULL;
    const char *Bid;
    long  naud=0, ndif=0;
    string diags, blockId;

    CifFile * fobjR = NULL;
    CifParser* cifParserR = NULL;
    fobjR = new CifFile();
    cifParserR = new CifParser(fobjR, fobjR->GetVerbose());
    cifParserR->Parse(infile, diags);
    delete(cifParserR);

    blockId = fobjR->GetFirstBlockName();
    Bid=blockId.c_str();
//    get_cifobj(infile, fobjR, Bid);
    
    aud_id=get_attribute_values(fobjR, Bid,"audit", "revision_id", &naud);
    aud_rec=get_attribute_values(fobjR, Bid,"audit", "update_record", &naud);
    aud_meth=get_attribute_values(fobjR, Bid,"audit", "creation_method", &naud);
    aud_date=get_attribute_values(fobjR, Bid,"audit", "creation_date", &naud);

    if(nblock==1)write_audit(fout, date, correct, naud, aud_id, aud_rec, aud_date,aud_meth, 1);

    
    dif_id=get_attribute_values(fobjR, Bid,"diffrn", "id", &ndif);
    dif_cid=get_attribute_values(fobjR, Bid,"diffrn", "crystal_id", &ndif);
    temp=get_attribute_values(fobjR, Bid,"diffrn", "ambient_temp", &ndif);
    treat=get_attribute_values(fobjR, Bid,"diffrn", "crystal_treatment", &ndif);
    dif_detail=get_attribute_values(fobjR, Bid,"diffrn", "details", &ndif);
    
    if(dif_cid || dif_detail || temp || treat || strlen(comment)>1){
        write_diffrn(fout, ndif, dif_id, dif_cid, dif_detail, temp, treat, comment);
    }
}

void write_diffrn(FILE *fout, long ndif, char **dif_id,char **dif_cid, 
                  char **dif_detail,char **temp, char **treat, char *comment)
{
    int i=0, k=1;
    

    fprintf(fout,"#\nloop_\n");
    fprintf(fout,"_diffrn.id \n");
    fprintf(fout,"_diffrn.crystal_id \n");
    fprintf(fout,"_diffrn.ambient_temp \n");
    fprintf(fout,"_diffrn.crystal_treatment \n");
    fprintf(fout,"_diffrn.details \n");
    
    for(i=0; i<ndif; i++){
        if(dif_id) {
            fprintf(fout,"'%s' ", dif_id[i]);
        }else{
            (i==0) ? fprintf(fout," 1 ") : fprintf(fout," ? ");
        }
            
        if(dif_cid) {
            fprintf(fout,"'%s' ", dif_cid[i]);
        }else{
            (i==0) ? fprintf(fout," 1 ") : fprintf(fout," ? ");
        }
            
        (temp) ? fprintf(fout,"'%s' ", temp[i]) : fprintf(fout," ? ");
        (treat) ? fprintf(fout," \"%s\" ", treat[i]) : fprintf(fout," ? ");

            
        (dif_detail) ? fprintf(fout," \n;%s\n;", dif_detail[i]) : fprintf(fout," ? ");
        fprintf(fout,"\n");
        
    }
    k=ndif+1;
    
    if (strlen(comment)>1){
        fprintf(fout,"%d 1 ? ? \n;%s\n; \n", k, comment);
    }
    

}


void write_audit(FILE *fout, char *date, char *correct,long naud,char **aud_id,
                 char **aud_rec,char **aud_date, char **aud_meth, int id)
{
    char tmp[256], tmp1[256], tmp2[256];
    int i, n=0, nt=0;
    
    fprintf(fout,"#\nloop_\n");
    fprintf(fout,"_audit.revision_id  \n");
    fprintf(fout,"_audit.creation_date  \n");
   // if(naud && aud_meth) fprintf(fout,"_audit.creation_method  \n");
    fprintf(fout,"_audit.update_record  \n");


    
    if (strlen(AUDIT)>0){
        nt=sscanf(AUDIT, "%s %s", tmp1, tmp2);
        n=strlen(tmp1);
        
        if (!(n==10 && !strncmp(tmp1, "20", 2))){
            printf("Error: wrong date (%s) format.(it should be yyyy-mm-dd)\n", tmp1);
        }
        if (nt==2){ //user added contents
            sscanf(AUDIT, "%s", date);
            strcpy(correct, get_substr(AUDIT,":", "@"));
        }else{
            strcpy(date, tmp1);
        }
        
    }
 

    n=0;
    if(naud){
        strcpy(tmp, "4_0");
        for(i=0; i<naud; i++){
            if(aud_id){
                fprintf(fout,"'%s' ", aud_id[i]);
                if (!strcmp(tmp, aud_id[i])){
                    n++;
                    sprintf(tmp, "4_%d", n);
                }
                
            }else{
                fprintf(fout," 1_0  ");
            }
            
            (aud_date) ? fprintf(fout,"'%s' ", aud_date[i]) : fprintf(fout," ? ");
          //  if (aud_meth) fprintf(fout,"\"%s\" ", aud_meth[i]) ;
            (aud_rec) ? fprintf(fout,"\n;%s\n;\n ", aud_rec[i]) : fprintf(fout," ? ");
                
            fprintf(fout,"\n");
        }
        
        if (strlen(AUDIT)>1 and id>0){ //add audit
            if (aud_meth){
                fprintf(fout,"%s  %s   \n;%s\n; \n", tmp,  date, correct);
            }else{
                fprintf(fout,"%s  %s  \n;%s\n; \n", tmp,  date, correct);
            }
            
            
        }
        
    }else{
        fprintf(fout," 1_0  ?  ?  \n");
        
        if (strlen(AUDIT)>1){
            fprintf(fout,"4_0  %s  \n;%s\n; \n",  date, correct);
        }
    }
    

}

        

void correct_list(char *inpf1)
/* correct a list of files (3 column in each row)
   pdbid, date(as 2012-02-26), "corrections"
 */
{
    char  infile[256], ofile[256], id1[20], correct[512], command[512];
    char **line1,  date[20],date_new[30], pdbfile[256];
    int  i, nline1;
    
    line1=get_lines_from_file(inpf1, &nline1);
    
    printf("Doing sf correction for the list (%s)\n", inpf1);
    
    for(i=0; i<nline1; i++){
        strcpy(date,"?");
        
        sscanf(line1[i],"%s%s", id1, date_new);
        strcpy(correct, get_substr(line1[i],"\"","\""));
        rid_of_front_end_space(correct);
               
        if(strlen(id1) != 4 ) continue;
        
        sprintf(infile, "r%ssf.ent.mmcif", id1);//the corrected file
        sprintf(ofile, "r%ssf.ent_new", id1);
        
        sprintf(command, "/bin/csh -c \" auto -pdb  %s \" ",id1);
        system(command);
        sprintf(pdbfile, "pdb%s.ent", id1);
        
//        make_sf_standard(infile, pdbfile, ofile, correct, '?', date_new, id1);
    
        printf("\nThe new file is %s (%s  %s %s)\n", ofile, id1,date_new, correct );
        
//        sprintf(command, "/bin/csh -c \" rm -f *_tmp? *_temp?; echo ===correcting %s  ===; diff  %s  %s \" ",infile,   infile, ofile);
//        system(command);
    }
    if(nline1 && line1) free_memory_2d(line1, nline1);
    

}


void correct_list_file(char *inpf1, char *inpf2)
{
    char  infile[256], ofile[256], id1[20], id2[20], correct[256], command[512];
    char **line1, **line2, date[20],date_new[30], pdbfile[256];
    int  i,j, nline1, nline2;

    strcpy(date_new, "2010-02-16"); //the new date for release

    
    line1=get_lines_from_file(inpf1, &nline1);
    line2=get_lines_from_file(inpf2, &nline2);
    
    printf("Doing sf correction (%d %d)\n", nline1,nline2);
    for(i=0; i<nline1; i++){
        strcpy(date,"?");
        
        sscanf(line1[i],"%s", id1);
        if(strlen(id1)<4 || id1[0] =='!' ) continue;
        
//        sprintf(infile, "r%ssf.ent_new", id1);
        sprintf(infile, "r%ssf.ent.mmcif", id1);//the corrected file
        sprintf(ofile, "r%ssf.ent_new", id1);
        
        sprintf(command, "/bin/csh -c \" auto -pdb  %s \" ",id1);
        system(command);
        sprintf(pdbfile, "pdb%s.ent", id1);
        
        strcpy(correct,line1[i]+5);
        rid_of_front_end_space(correct);

        for(j=0; j<nline2; j++){ //look for date of the ID
            sscanf(line2[j],"%s", id2);
            if(!strcmp_case(id1, id2)){
                sscanf(line2[j],"%*s%*s%s", date);
                break;
            }
        }
        
        make_sf_standard(infile, pdbfile, ofile, correct, date,date_new, id1);
        
    
        printf("\nThe new file is %s\n", ofile);
        
//        sprintf(command, "/bin/csh -c \" rm -f *_tmp? *_temp?; echo ===correcting %s  ===; diff  %s  %s \" ",infile,   infile, ofile);
//        system(command);
    }
    if(nline1 && line1) free_memory_2d(line1, nline1);
    if(nline2 && line2) free_memory_2d(line2, nline2);
    

}



void update_audit_new(char *infile,  char *date, char *date_new, char *correction)
{
    char  **line, **aud_date=NULL,**aud_rec=NULL, **aud_id=NULL;
    int id=0, nline, i, j;
    long  naud=0;
    string diags;

    FILE *fout;
    


//    sprintf(outf, "%s_audit", infile);
    printf("Updating audit record(%s)...\n", infile);
    
    
    cifparse(infile, infile, 100) ;
    line=get_lines_from_file(infile, &nline);
    
    
    for(i=0; i<nline; i++){
        rid_of_front_space(line[i]);
        if(!strncmp_case(line[i], "_audit.", 7)) id=1;
    }

    
    if(id>0){
        CifFile * fobjR = NULL;
        CifParser* cifParserR = NULL;
        fobjR = new CifFile();
        cifParserR = new CifParser(fobjR, fobjR->GetVerbose());
        cifParserR->Parse(infile, diags);
        delete(cifParserR);
        const char *Bid;
        string blockId;

        blockId = fobjR->GetFirstBlockName();
    
        Bid=blockId.c_str();
        aud_id=get_attribute_values(fobjR, Bid,"audit", "revision_id", &naud);
        aud_date=get_attribute_values(fobjR, Bid,"audit", "creation_date", &naud);
        aud_rec=get_attribute_values(fobjR, Bid,"audit", "update_record", &naud);
        
    }
    fout=fopen(infile, "w");
    
    for(i=0; i<nline; i++){
        if(!strncmp(line[i],"data_", 5)){
            fprintf(fout,"%s", line[i]);
            
            fprintf(fout,"#\nloop_\n");
            fprintf(fout,"_audit.revision_id  \n");
            fprintf(fout,"_audit.creation_date  \n");
            fprintf(fout,"_audit.update_record  \n");

            if(aud_date) {
                for(j=0; j<naud; j++){
                    fprintf(fout,"1_%d  ", j);
                    
                    if (j==0 && aud_date[j][0]=='?'){
                        fprintf(fout," %s ", date );
                    }else{
                        fprintf(fout," %s ", aud_date[j]);
                    }
                    
                    if(j==0 && !aud_rec){
                        fprintf(fout," 'Initial release' \n"); 
                    }else{
                        (aud_rec) ? fprintf(fout," '%s' \n", aud_rec[j]) : fprintf(fout," ? \n");
                    }
                }
                fprintf(fout,"1_%d  %s  '%s' \n", j, date_new, correction);
                
            }else{
                fprintf(fout,"1_0  %s  'Initial release' \n",  date);
                fprintf(fout,"1_1  %s  '%s' \n", date_new, correction);
            }
            
        }else if((strstr(line[i],"loop_") && strstr(line[i+1],"_audit.")) ||
                 strstr(line[i],"_audit.")){//skip audit
            for(j=i; j<nline; j++){
                rid_of_front_space(line[j]);
                if (line[j][0]=='#'){
                    i=j;
                    break;
                }
                continue;
            }
        }else{
            fprintf(fout,"%s", line[i]);
        }
    }
    fclose(fout);

}


void make_sf_standard(char *sffile, char *pdbfile ,char *ofile ,
                      char *correct, char *date, char *date_new,char* pdb_id)
{
    char outf_tmp1[256], outf_tmp2[256], outf_tmp3[256] , **line;
    int i,j, nline, no=0, nf=0, ns=0;
    FILE  *fw,  *fw1, *fw2;
   
    sprintf(outf_tmp1, "%s_temp1", sffile);
    sprintf(outf_tmp2, "%s_temp2", sffile);
    sprintf(outf_tmp3, "%s_temp3", sffile);

    fw=fopen(ofile, "w");
    fw1=fopen(outf_tmp1, "w");
    fw2=fopen(outf_tmp2, "w");
        
    line=get_lines_from_file(sffile, &nline); //seperate 1st and 2st parts
    for(i=0; i<nline; i++){
        if(strstr(line[i],"loop_") && strstr(line[i+1],"_refln.")){
            fprintf(fw2,"data_r%ssf\n", pdb_id);
            for(j=i; j<nline; j++){
                fprintf(fw2,"%s",line[j]);
            }
            i=j;
            
        }else{
            fprintf(fw1,"%s",line[i]);
        }
        
    }
    fclose(fw1);
    fclose(fw2);
    if(line && nline) free_memory_2d(line,nline);
     
    cifparse(outf_tmp1, outf_tmp1, 100) ;
    make_complete_SF(pdbfile, outf_tmp1);
    update_audit_new(outf_tmp1, date, date_new, correct);

    
    line=get_lines_from_file(outf_tmp1, &nline);
    for(i=0; i<nline; i++){
        fprintf(fw, "%s", line[i]);
    }
    if(line && nline) free_memory_2d(line,nline);


    line=get_lines_from_file(outf_tmp2, &nline);
    for(i=0; i<nline; i++){
        if(strstr(line[i], " o")){
            no++;
        }else if(strstr(line[i], " f")){
            nf++;
        }else if (strstr(line[i], "_refln.status")){
            ns=1;
        }else if(i>100 && strstr(line[i], "data_")){
            break;
        }
    }
    if(line && nline) free_memory_2d(line,nline);
    

    if(ns==1 && (no<200 || (nf>0 && no<nf))){ //status exist
        cif2cif_sf_all(outf_tmp2, outf_tmp3, pdb_id, 1);
        line=get_lines_from_file(outf_tmp3, &nline);
    }else{
        cifparse(outf_tmp2, outf_tmp2, 100) ;
        line=get_lines_from_file(outf_tmp2, &nline);
    }
    

    for(i=0; i<nline; i++){
        if(i<10 && strstr(line[i], "data_")) continue;
        fprintf(fw, "%s", line[i]);
    }
    fprintf(fw, "#END OF REFLECTIONS\n");
    if(line && nline) free_memory_2d(line,nline);
    fclose(fw);
}
 

/*            
void insert_column(char *infile, const char *token)
{
    char  **line;
    int i,j,id=0, nline=0;
    FILE *fw=NULL;
    

    line=get_lines_from_file(infile, &nline);

    for(i=0; i<nline; i++){
        if (strstr_case(line[i], token)) {
            id=1;
            printf("Warning! The %s is not in %s and will be inserted as 1\n", token,infile );
            break;
        }
        if(i>400) break;
    }

    if(id <=0) return;

    fw=fopen(infile, "w");
    for(i=0; i<nline; i++){
        fprintf(fw,"%s\n", line[i]);
        
        if(strstr_case(line[i], "loop_") && strstr_case(line[i+1], "_refln.") ){
            fprintf(fw,"%s\n", token);
            fprintf(fw,"%s", line[i+1]);
            for(j=i+2; j<nline; j++){
                if(strchr(line[j], '#')) break;
                if( !strstr_case(line[j], "_refln.")){
                    fprintf(fw, " 1 %s", line[j]);
                }else{
                    fprintf(fw, "%s", line[j]);
                }
            }
        }
    }
    fclose(fw);
    
    if(nline) free_memory_2d(line,nline);
    

}
*/


void add_audit(char *outfile)
{

    char **line;
    int i, nline=0, n=0;
    FILE *fw=NULL;
    
    line=get_lines_from_file(outfile, &nline);
    
    
    for(i=0; i<nline; i++){
        if(strstr_case(line[i], "_audit.")){
            n=1;
            break;
        }
    }

    if(n==0){
        fw=fopen(outfile, "w");
        for(i=0; i<nline; i++){
            if(strstr(line[i],"data_")){
                fprintf(fw,"%s",line[i]);
                audit_sf_header(fw,0);
            }else{
                fprintf(fw,"%s",line[i]);
            }
        }
        fclose(fw);
    }
    if(nline)free_memory_2d(line,nline);
    
}


void update_audit(char *inpf)
// given a list containing pdbid and corrections
{
    char **line, infile[256], ofile[256], id[20], correct[256], command[512];
    int nline=0, i;

    line=get_lines_from_file(inpf, &nline);
    
    for(i=0; i<nline; i++){
        
        strncpy(id,line[i], 4);
        id[4]='\0';
        rid_of_space(id);
        if(strlen(id)<4 || id[0] =='!' ) continue;
        
        sprintf(infile, "r%ssf.ent_new", id);
        sprintf(ofile, "r%ssf.ent_new_audit", id);
        strcpy(correct,line[i]+5);
        rid_of_front_end_space(correct);
        
        add_items_to_cif(infile, ofile, correct);
        
//        reformat_file(ofile, ofile);
    
        printf("\nThe new file is %s\n", ofile);
        
        sprintf(command, "/bin/csh -c \" rm -f *_tmp?; echo ===correcting %s  ===; diff  %s  %s \" ",infile,   infile, ofile);
        system(command);
    }

}

void add_items_to_cif(char *inpfile, char *outfile, char *correct)
{
    char  **line, str[512], ftmp1[512], ftmp2[512], cate[80], item[80];
    char **aud_date=NULL,**aud_rec=NULL, date[80] ;
    long  naud=0, na=0;
    int nline, i,j=0,key=0;
    string diags;
    FILE *fout=NULL, *fo1=NULL, *fo2=NULL;


    strcpy(date, "2009-09-01");

//    sprintf(inpfile, "%s_inp_tmp1", inpfile_inp);, inpfile[512]
//    reformat_file(inpfile_inp, inpfile);
    
    line=get_lines_from_file(inpfile, &nline);
    
    sprintf(ftmp1, "%s_tmp1", inpfile);
    sprintf(ftmp2, "%s_tmp2", inpfile);
    
    fout=fopen(outfile,"w");
    fo1=fopen(ftmp1,"w");
    fo2=fopen(ftmp2,"w");

    
    
    for(i=0; i<nline; i++){

        if((strstr(line[i],"loop_") && strstr(line[i+1],"_audit.") && i<nline-2 && key==0) ||
           (strstr(line[i],"_audit.") && key==0) ){
            fprintf(fo1,"data_tmp\n");
            
            for(j=i; j<nline; j++){
                rid_of_front_space(line[j]);
                if(line[j][0]=='#' || (!strncmp(line[j],"loop_", 5)&& j>i) ||
                   (check_category(line[j],cate, item) && !strstr(cate, "audit") )) {
                    fprintf(fo2,"%s", line[j]);
                    i=j+1;
                    
                    break;
                    
                }
                fprintf(fo1,"%s", line[j]);
                na++;
                
            }
            key=1;
            
        }
        
        fprintf(fo2,"%s", line[i]);
    }
    fclose(fo1); fclose(fo2);
    
    if(nline) free_memory_2d(line, nline);
    
    if(na>0){
        CifFile * fobjR = NULL;
        CifParser* cifParserR = NULL;
        fobjR = new CifFile();
        cifParserR = new CifParser(fobjR, fobjR->GetVerbose());
        cifParserR->Parse(ftmp1, diags);
        delete(cifParserR);
        const char *Bid;
        string blockId;

        blockId = fobjR->GetFirstBlockName();
    
        Bid=blockId.c_str();
//        aud_id=get_attribute_values(fobjR, Bid,"audit", "revision_id", &naud);
        aud_date=get_attribute_values(fobjR, Bid,"audit", "creation_date", &naud);
        aud_rec=get_attribute_values(fobjR, Bid,"audit", "update_record", &naud);
    }
    
    nline=0;
    
    line=get_lines_from_file(ftmp2, &nline);
//    printf("the new file is %s:: %d\n", ftmp2,nline );

    
    for(i=0; i<nline; i++){
        strcpy(str, line[i]);
        rid_of_front_space(str);

        if(!strncmp(str,"data_", 5) && i<200){
            fprintf(fout,"%s", line[i]);
            
//            if(na>0){
                fprintf(fout,"#\nloop_\n");
                fprintf(fout,"_audit.revision_id  \n");
                fprintf(fout,"_audit.creation_date  \n");
                fprintf(fout,"_audit.update_record  \n");
                for(j=0; j<naud; j++){
                     fprintf(fout,"1_%d  ", j);
                    if(aud_date) {
                        rid_of_space(aud_date[j]);
                        fprintf(fout,"%s  ", aud_date[j]) ;
                    }else
                        fprintf(fout," ? ");
                     
                    (aud_rec) ? fprintf(fout,"'%s' ", aud_rec[j]) : fprintf(fout," ? ");
                    fprintf(fout,"\n");
                }
                fprintf(fout,"1_%d  %s  '%s' \n", j, date, correct);
//            }
            
        }else {
            fprintf(fout,"%s", line[i]);
        }
    }
    fclose(fout);
    if(nline)free_memory_2d(line, nline);
    
    
}

