/* convert a structure factor file from various format to mmCIF format
   not include mmcif --> mmcif!
*/
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <ctype.h>
#include <sys/types.h>
#include <unistd.h>
#include "rcsb_math.h"
#include "sf_convert.h"
#include "util.h"
 
//extern int TEST;
extern char *CCP4, MTZ2CIF[512], *PATHTMP, FREERV[20];
extern int ADD_MAP;
int NFIRST=0;

void mtzdmp_file(char *inpfile, char *outfile, long nref);
   
void various_2_cif(char *inpfile, char *outfile, char *format_inp,
                   char *pdb_id, char *type)
{
    int n=0;
    FILE *finp=NULL, *fout =NULL;
     
    
    if((finp=fopen(inpfile,"r"))==NULL){
        open_file_error(inpfile,"various_2_cif");
    }
    fclose(finp);
    
    
    if((fout=fopen(outfile,"w"))==NULL){
        printf("Error! No written permission for the file %s!\n", outfile);
        exit(0);
    }
    
    if(strcmp_case(format_inp,"MTZ") && strcmp_case(format_inp,"mmcif")){
        fprintf(fout, "data_r%ssf\n", pdb_id);
    }
           
    if(!strcmp_case(format_inp, "CNS") || !strcmp_case(format_inp, "CNX")||
       !strcmp_case(format_inp, "XPLOR")|| !strcmp_case(format_inp, "MAIN")){
        cns2mmcif(fout,  inpfile);
        
    }else if (!strcmp_case(format_inp, "SHELX") ){
        n = is_small_mol_cif(inpfile);
        if(n>0){ //small molecule
            printf("The input format is small molecule CIF.\n");
            cif2mmcif(fout,inpfile);
        }else
            shelx2mmcif(fout,inpfile,type );
        
    }else if (!strcmp_case(format_inp, "XPREP")||!strcmp_case(format_inp, "SAINT") ){
        saint2mmcif(fout,inpfile);
        
    }else if (!strcmp_case(format_inp, "fi") ){
        
        fi2mmcif(fout,inpfile, NULL);
//convert reflection file (3I4,2F8.2,2F8.2,I4 for SHELX ) to CIF
        
    }else if(!strncmp_case(format_inp, "HKL",3)|| !strcmp_case(format_inp,"scalepack") ||
            !strcmp_case(format_inp, "3DSCALE") ) {
        hkl2mmcif(fout,inpfile);
        
    }else if(!strcmp_case(format_inp, "EPMR") || !strcmp_case(format_inp, "OTHER")) {
        epmr2mmcif(fout,inpfile, type);
        
    }else if (!strcmp_case(format_inp, "TNT")){
        tnt2mmcif(fout,inpfile);
        
    }else if (!strcmp_case(format_inp, "XSCALE") ||!strcmp_case(format_inp, "XDS") ){
        xscale2mmcif(fout,inpfile);

    }else if (!strcmp_case(format_inp, "MTZ")){
        mtz2mmcif(inpfile, outfile); 
        reformat_mtz_output(outfile,pdb_id);
        
    }else if(!strcmp_case(format_inp, "DTREK") ) {
        dtrek2mmcif(fout,inpfile);
        
        
    }else if(!strcmp_case(format_inp, "XTALVIEW") ) {
        xtalview2mmcif(fout,inpfile);
        
    }else if(!strcmp_case(format_inp, "X-GEN") || !strcmp_case(format_inp, "XENGEN") ) {
        xgen2mmcif(fout,inpfile, type);
        
    }else if(!strcmp_case(format_inp, "MULTAN") ) {
        multan2mmcif(fout,inpfile, type);
        
    }else if(!strcmp_case(format_inp, "CIF") ) {
        cif2mmcif(fout,inpfile);
        
    }else {
        printf("Error! The input file format (%s) is wrong!\n",format_inp );
        usage(0);
    }
    if(strcmp_case(format_inp, "MTZ")) fprintf(fout, "#END OF REFLECTIONS\n");
    fclose(fout);

    check_mmcif_status(outfile);
    
    
}

void check_mmcif_status(char *outfile)
// if status is wrong, reverse it.
{
    char **lines=NULL;
    int i, nline, n=0, j=0, m,k,len;
    FILE *fout=NULL;
    
    lines=get_lines_from_file(outfile, &nline);
    for(i=0; i<nline-5; i++){ 
        if(strstr(lines[i+0],"loop_") &&
           strstr(lines[i+1],"_refln.") &&
           strstr(lines[i+2],"_refln.") ){
            n=1;
            j=i;
            
            break;
        }
        if(i>1000) break;
    }
    
    if(n<=0 || nline<50){
        printf("Error! conversion to mmcif was not successful!\n");
        if(nline>0) free_memory_2d(lines, nline);
        return;
    }

    n=0;
    m=0;
    for(i=j; i<nline; i++){  //get cif items
        if(strstr(lines[i], " o"))
            n++;
        else if(strstr(lines[i], " f"))
            m++;
    }
    if(m>n+100) { // more flag than observed, rewrite
        fout=fopen(outfile, "w");
        
        for(i=0; i<nline; i++){
            if(strstr(lines[i+0],"loop_") &&
               strstr(lines[i+1],"_refln.") &&
               strstr(lines[i+2],"_refln.") ){
                j=i;
                break;
            }
            fprintf(fout,"%s",lines[i]);
        }

        for(i=j; i<nline; i++){
            len=strlen(lines[i]);
            
            for(k=1; k<len; k++){
                if(lines[i][k]=='o' && lines[i][k-1] ==' ')
                    lines[i][k]='f';
                else if(lines[i][k]=='f' && lines[i][k-1] ==' ')
                    lines[i][k]='o';
            }
            
            fprintf(fout,"%s",lines[i]);
        }
        fclose(fout);
    }
    
}


int is_small_mol_cif(char *inpfile)
{
    char **lines;
    int nline, i, n=0;


    lines=get_lines_from_file(inpfile, &nline);
    
    
    for(i=0; i<nline-5; i++){  //get cif items
        rid_of_front_end_space(lines[i+0]);
        rid_of_front_end_space(lines[i+1]);
        rid_of_front_end_space(lines[i+2]);
        rid_of_front_end_space(lines[i+3]);
        
        if(!strncmp(lines[i+0],"_refln_",7) &&
           !strncmp(lines[i+1],"_refln_",7) &&
           !strncmp(lines[i+2],"_refln_",7) &&
           !strncmp(lines[i+3],"_refln_",7)){
            n=1;
            break;
        }
        if(i>1000) break;
        
    }

    if(nline>0) free_memory_2d(lines, nline);
    return n;
}


void audit_sf_header(FILE *fout, int id)

{
    if(id==0){
        fprintf(fout,"#\n%s\n%s\n%s\n#\n",
                "_audit.revision_id      1_0 ",
                "_audit.creation_date  ?",
                "_audit.update_record    'Initial release' "
                );
    }else{
        fprintf(fout,"#\n%s\n%s\n%s\n",
                "_audit.revision_id      ",
                "_audit.creation_date  ",
                "_audit.update_record    "
                );
    }
    
    
}


              
int xplor2mmcif(FILE *ftmp, FILE *fout, int nfg, char *flag_cif)
{
    char str[512];
    int i=0, id=0, H,K,L;
    float F=0,sF=0,I=0,sI=0;
    
    
    
    rewind(ftmp);
    
    while (fgets(str, sizeof str, ftmp) != NULL) { 
        if(strstr_case(str,"INDE") &&
           (strstr_case(str," FOBS ") || strstr_case(str," IOBS "))){
            id=1;
            break;
        }
    }
    if(id==0) return 0; //CNS format
    printf("Converting the Xplor format\n");
    
    rewind(ftmp);
    while (fgets(str, sizeof str, ftmp) != NULL) { 
        if(strstr_case(str,"INDE")){
            F=0;
            sF=0;
            I=0;
            sI=0;
            sscanf(str,"%*s %d%d%d",&H,&K,&L);

            if(strstr_case(str, " FOBS") && strstr_case(str, " IOBS")){//both I&F
                F= get_float_after_id(str, "FOBS");
                sF=get_float_after_id(str, "SIGMA");
                I= get_float_after_id(str, "IOBS");
                sI=get_float_after_id(str, "SIGI");
                
                if(nfg>0){
                    fprintf(fout,"1 1 1 %4d %4d %4d %10.2f %10.2f %10.2f %10.2f %c \n", 
                            H,K,L, F, sF, I, sI, flag_cif[i]);
                }else
                    fprintf(fout,"1 1 1 %4d %4d %4d %10.2f %10.2f %10.2f %10.2f \n", 
                            H,K,L, F, sF, I, sI);
                i++;
                
            }else if(strstr_case(str, " FOBS") && !strstr_case(str, " IOBS")){//only F
                F= get_float_after_id(str, "FOBS");
                sF=get_float_after_id(str, "SIGMA");
                
                if(nfg>0){
                    fprintf(fout,"1 1 1 %4d %4d %4d %10.2f %10.2f  %c \n", 
                            H,K,L, F, sF, flag_cif[i]);
                }else
                    fprintf(fout,"1 1 1 %4d %4d %4d %10.2f %10.2f \n", 
                            H,K,L, F, sF);
                i++;
               
                
            }else if(!strstr_case(str, " FOBS") && strstr_case(str, " IOBS")){ //only I
                I= get_float_after_id(str, "IOBS");
                sI=get_float_after_id(str, "SIGI");
                
                if(nfg>0){
                    fprintf(fout,"1 1 1 %4d %4d %4d %10.2f %10.2f %c \n", 
                            H,K,L, I, sI, flag_cif[i]);
                }else
                    fprintf(fout,"1 1 1 %4d %4d %4d %10.2f %10.2f \n", 
                            H,K,L, I, sI);
                i++;
            }
            
        }
    }
    return 1;
    


}

void cns2mmcif(FILE *fout,  char *inpfile)
/* Convert the CNS reflection data to mmCIF */
{
    char str[512],  tmp[512],  *flag_cif=NULL ;
    long  i, in=0,n=0, fn=0,  nref=0, H, K, L, nfg=0;
    float  F, sF, I, sI, pa,pb, pc, pd,ffg=0;
    FILE *finp, *ftmp;
    
    if((finp = fopen(inpfile, "r"))==NULL) {
        open_file_error(inpfile, "cns2mmcif");
    }
    
    ftmp = tmpfile();
    while (fgets(str, sizeof str, finp) != NULL) {
        sscanf(str,"%s",tmp);
        if(strstr_case(tmp, "DECLare")){
            if(pattern5(str,"DECLare", "NAME=IOBS", "DOM", "", "") ){
                in=1;
            }else if(pattern5(str,"DECLare", "NAME=FOBS", "DOM", "", "") ||
                     pattern5(str,"DECLare", "NAME=F_NAT", "DOM", "", "")){
                fn=1;
            }
        }else{
            if( strstr_case(str, " IOBS ") || strstr_case(str, " IOBS=") ||
                strstr_case(str, " I_")){
                in=1;
            }
            if(strstr_case(str, " FOBS ") || strstr_case(str, " FOBS=") ||
               strstr_case(str, " F_") ){
                fn=1;
            }
            
            if(strstr_case(tmp, "INDE")){ //merge lines for the same INDEX
                fprintf(ftmp, "\n");
                nref++;
            }
            n=strlen(str);
            for(i=0; i<n; i++){
                if(isspace(str[i])) str[i]=' '; 
                fprintf(ftmp, "%c", str[i]);
            }
        }
    }
    
    if(nref < 50  || (in ==0 && fn ==0) ){
        printf("Error! Can not find IOBS or FOBS in your file (%s).\n", inpfile);
        return;
    }
    
    flag_cif = new char[nref+10];

    nfg=0;
    rewind(ftmp);
    while (fgets(str, sizeof str, ftmp) != NULL) {
        
        if(strstr_case(str,"INDE")){
            
            if(strstr_case(str, "TEST") ||strstr_case(str, "FREE")){
                if(strstr_case(str, "TEST=")){
                    ffg=get_int_after_id(str, "TEST=");
                }else if(strstr_case(str, "TEST")){
                    ffg=get_int_after_id(str, "TEST ");
                }else if(strstr_case(str, "FREE=")){
                    ffg=get_int_after_id(str, "FREE=");
                }else if(strstr_case(str, "FREE ")){
                    ffg=get_int_after_id(str, "FREE ");
                }
                if (strlen(FREERV)>0  && ffg == atof(FREERV)) ffg=1;

                (ffg==1) ? flag_cif[nfg] = 'f' : flag_cif[nfg] = 'o';
                nfg++;
            }
        }
    }
    
    if(in>0 &&  fn>0 ){ //both exist   
        if(nfg>0)
            sf_data_header(fout, 0, 1);
        else
            sf_data_header(fout, 0, 0);
    }else if(in>0 &&  fn==0 ){ //only intensity exists        
        if(nfg>0)
            sf_data_header(fout, 2, 1);
        else
            sf_data_header(fout, 2, 0);
    }else if(fn>0 &&  in==0 ){ //only amplitude exists        
        if(nfg>0)
            sf_data_header(fout, 1, 1);
        else
            sf_data_header(fout, 1, 0);
    }        
    
    i=0;

    if( xplor2mmcif(ftmp, fout, nfg, flag_cif )) return;
    
    rewind(ftmp);
    while (fgets(str, sizeof str, ftmp) != NULL) { 
        if(strstr_case(str,"INDE")){
            F=0;
            sF=0;
            I=0;
            sI=0;
            sscanf(str,"%*s %ld%ld%ld",&H,&K,&L);

            
            if((strstr_case(str, "PA_MOD=") && strstr_case(str, "PB_MOD=") &&
                strstr_case(str, "PC_MOD=") && strstr_case(str, "PD_MOD="))||
               
               (strstr_case(str, "PA_") && strstr_case(str, "PB_") &&
                strstr_case(str, "PC_") && strstr_case(str, "PD_"))||
               
               (strstr_case(str, "PA=") && strstr_case(str, "PB=") &&
                strstr_case(str, "PC=") && strstr_case(str, "PD="))
               ){
                if(i==0){
                    fprintf(fout,"_refln.pdbx_HL_A_iso\n");
                    fprintf(fout,"_refln.pdbx_HL_B_iso\n");
                    fprintf(fout,"_refln.pdbx_HL_C_iso\n");
                    fprintf(fout,"_refln.pdbx_HL_D_iso\n");
                }
            }
            
            if(strstr_case(str, " FOBS=") && strstr_case(str, " IOBS=")){//both I&F
                
                strcpy(tmp, strstr_case(str, "FOBS=")+5);
                if(strlen(tmp)>0) sscanf(tmp,"%f", &F);

                if(!strstr(str, "SIGMA=")){
                    printf("Sorry! the Sigma for FOBS item is not SIGMA\n");
                    printf("Please change it to SIGMA and try again.\n");
                    return;
                }
                    
                strcpy(tmp, strstr_case(str, "SIGMA=")+6);
                if(strlen(tmp)>0) sscanf(tmp,"%f", &sF);
                
                strcpy(tmp, strstr_case(str, "IOBS=")+5);
                if(strlen(tmp)>0) sscanf(tmp,"%f", &I);
                
                if(strstr(str, "SIGI=")){
                    strcpy(tmp, strstr_case(str, "SIGI=")+5);
                    sscanf(tmp,"%f", &sI);
                }else if(strstr(str, "ISIGMA=")){
                    strcpy(tmp, strstr_case(str, "ISIGMA=")+7);
                    sscanf(tmp,"%f", &sI);
                    
                }else if(strstr(str, "SIGMAI=")){
                    strcpy(tmp, strstr_case(str, "SIGMAI=")+7);
                    sscanf(tmp,"%f", &sI);
                }else{
                    printf("Sorry! the Sigma data item is not SIGI or ISIGMA or SIGMAI.\n");
                    printf("Please change it to either of them, and try again.\n");
                    return;
                }
                
                if(nfg>0){
                    fprintf(fout,"1 1 1 %4ld %4ld %4ld %10.2f %10.2f %10.2f %10.2f %c ", 
                            H,K,L, F, sF, I, sI, flag_cif[i]);
                }else
                    fprintf(fout,"1 1 1 %4ld %4ld %4ld %10.2f %10.2f %10.2f %10.2f ", 
                            H,K,L, F, sF, I, sI);
                i++;
                
            }else if(!strstr_case(str, " FOBS=") && strstr_case(str, " IOBS=")){ //only I
                
                strcpy(tmp, strstr_case(str, "IOBS=")+5);
                sscanf(tmp,"%f", &I);
                
                if(strstr_case(str, "SIGI=")){
                    strcpy(tmp, strstr_case(str, "SIGI=")+5);
                    sscanf(tmp,"%f", &sI);
                    
                }else if(strstr_case(str, "ISIGMA=")){
                    strcpy(tmp, strstr_case(str, "ISIGMA=")+7);
                    sscanf(tmp,"%f", &sI);
                    
                }else if(strstr_case(str, "SIGMAI=")){
                    strcpy(tmp, strstr_case(str, "SIGMAI=")+7);
                    sscanf(tmp,"%f", &sI);
                    
                }else 
                    sI=0;
                
                 if(nfg>0){
                    fprintf(fout,"1 1 1 %4ld %4ld %4ld %10.2f %10.2f %c ", 
                            H,K,L, I, sI, flag_cif[i]);
                }else
                    fprintf(fout,"1 1 1 %4ld %4ld %4ld %10.2f %10.2f ", 
                            H,K,L, I, sI);
               
                i++;
                
            }else if(strstr_case(str, " FOBS=") && !strstr_case(str, " IOBS=")){//only F
                
                strcpy(tmp, strstr_case(str, "FOBS=")+5);
                sscanf(tmp,"%f", &F);
                
                if(strstr_case(str, "SIGMA=")){
                    strcpy(tmp, strstr_case(str, "SIGMA=")+6);
                    sscanf(tmp,"%f", &sF);
                }else 
                    sF=0;

                 if(nfg>0){
                    fprintf(fout,"1 1 1 %4ld %4ld %4ld %10.2f %10.2f  %c ", 
                            H,K,L, F, sF, flag_cif[i]);
                }else
                    fprintf(fout,"1 1 1 %4ld %4ld %4ld %10.2f %10.2f ", 
                            H,K,L, F, sF);
                i++;
            }else if(strstr_case(str, " F_NAT") && !strstr_case(str, " IOBS=")){//only F
                
                strcpy(tmp, strstr_case(str, "F_NAT")+6);
                sscanf(tmp,"%f", &F);
                
                if(strstr_case(str, "S_NAT=")){
                    strcpy(tmp, strstr_case(str, "S_NAT=")+6);
                    sscanf(tmp,"%f", &sF);
                }else 
                    sF=0;

                 if(nfg>0){
                    fprintf(fout,"1 1 1 %4ld %4ld %4ld %10.2f %10.2f  %c ", 
                            H,K,L, F, sF, flag_cif[i]);
                }else
                    fprintf(fout,"1 1 1 %4ld %4ld %4ld %10.2f %10.2f ", 
                            H,K,L, F, sF);
                i++;
                
            }else if(strstr_case(str, " F_1") && !strstr_case(str, " IOBS=")){//only F
                
                strcpy(tmp, strstr_case(str, "F_1")+6);
                sscanf(tmp,"%f", &F);
                
                if(strstr_case(str, "S_1=")){
                    strcpy(tmp, strstr_case(str, "S_1=")+4);
                    sscanf(tmp,"%f", &sF);
                }else 
                    sF=0;

                 if(nfg>0){
                    fprintf(fout,"1 1 1 %4ld %4ld %4ld %10.2f %10.2f  %c ", 
                            H,K,L, F, sF, flag_cif[i]);
                }else
                    fprintf(fout,"1 1 1 %4ld %4ld %4ld %10.2f %10.2f ", 
                            H,K,L, F, sF);
                i++;
            }
            
            if(strstr_case(str, "PA_MOD=") && strstr_case(str, "PB_MOD=") &&
               strstr_case(str, "PC_MOD=") && strstr_case(str, "PD_MOD=")){
            
                strcpy(tmp, strstr_case(str, "PA_MOD=")+7);
                sscanf(tmp,"%f", &pa);
                strcpy(tmp, strstr_case(str, "PB_MOD=")+7);
                sscanf(tmp,"%f", &pb);
                strcpy(tmp, strstr_case(str, "PC_MOD=")+7);
                sscanf(tmp,"%f", &pc);
                strcpy(tmp, strstr_case(str, "PD_MOD=")+7);
                sscanf(tmp,"%f", &pd);
                fprintf(fout," %8.3f  %8.3f  %8.3f  %8.3f \n", pa, pb, pc, pd);
                
            }else if(strstr_case(str, "PA=") && strstr_case(str, "PB=") &&
               strstr_case(str, "PC=") && strstr_case(str, "PD=")){
            
                strcpy(tmp, strstr_case(str, "PA=")+3);
                sscanf(tmp,"%f", &pa);
                strcpy(tmp, strstr_case(str, "PB=")+3);
                sscanf(tmp,"%f", &pb);
                strcpy(tmp, strstr_case(str, "PC=")+3);
                sscanf(tmp,"%f", &pc);
                strcpy(tmp, strstr_case(str, "PD=")+3);
                sscanf(tmp,"%f", &pd);
                fprintf(fout," %8.3f  %8.3f  %8.3f  %8.3f \n", pa, pb, pc, pd);
                
            }else if(strstr_case(str, "PA_1=") && strstr_case(str, "PB_1=") &&
               strstr_case(str, "PC_1=") && strstr_case(str, "PD_1=")){
            
                strcpy(tmp, strstr_case(str, "PA_1=")+5);
                sscanf(tmp,"%f", &pa);
                strcpy(tmp, strstr_case(str, "PB_1=")+5);
                sscanf(tmp,"%f", &pb);
                strcpy(tmp, strstr_case(str, "PC_1=")+5);
                sscanf(tmp,"%f", &pc);
                strcpy(tmp, strstr_case(str, "PD_1=")+5);
                sscanf(tmp,"%f", &pd);
                fprintf(fout," %8.3f  %8.3f  %8.3f  %8.3f \n", pa, pb, pc, pd);
                
            }else if(strstr_case(str, "PA_CONV=") && strstr_case(str, "PB_CONV=") &&
               strstr_case(str, "PC_CONV=") && strstr_case(str, "PD_CONV=")){
            
                strcpy(tmp, strstr_case(str, "PA_CONV=")+8);
                sscanf(tmp,"%f", &pa);
                strcpy(tmp, strstr_case(str, "PB_CONV=")+8);
                sscanf(tmp,"%f", &pb);
                strcpy(tmp, strstr_case(str, "PC_CONV=")+8);
                sscanf(tmp,"%f", &pc);
                strcpy(tmp, strstr_case(str, "PD_CONV=")+8);
                sscanf(tmp,"%f", &pd);
                fprintf(fout," %8.3f  %8.3f  %8.3f  %8.3f \n", pa, pb, pc, pd);
                
                
            }else{
                fprintf(fout,"\n");
            }
                
        }
        
    }
    if(nref>0) delete [] flag_cif;
    if(finp) fclose(finp);
    if(ftmp) fclose(ftmp);
    
}

void check_flag_num(float *flag, long nref, const char *id, char *flag_cif)
{
    long i, nf=0, no=0, yes=0;

    for(i=0; i<nref; i++){
        if(!strcmp_case(id,"CNS")){
            if(flag[i] > 0.5 )
                nf++;
            else 
                no++;
        }else if(!strcmp_case(id,"SHELX")){
            if(flag[i] < -0.5 )
                nf++;
            else 
                no++;
        }
    }

    if(nf>10 && no >50){
//            printf("%7ld, %7ld, flag=%.3lf\n",nf, no, float(nf)/float(no));
        
        if(float(nf)/float(no) > 0.03){
            yes = 1;
        }else{
            yes = 0;
        }
    }else{
        yes = 0;
    }

    if(yes==1){ // use flag in the file
        for(i=0; i<nref; i++){
            flag_cif[i]='o';
            if(!strcmp_case(id,"CNS")){
                if(flag[i] > 0.5 )
                    flag_cif[i] = 'f';
                else 
                    flag_cif[i] = 'o';
            }else if(!strcmp_case(id,"SHELX")){
                
                if(flag[i] < -0.5 )
                    flag_cif[i] = 'f';
                else
                    flag_cif[i] = 'o';
            }
        }
        
    }else{ // generate 10% 
        

        for(i=0; i<nref; i++){
            flag_cif[i]='o';
        }

    }
}

void formated_4_8(char *str, int *h, int *k, int *l, float *a,float *sa)
{
    char tmp[10];
    int  H, K, L;
    float sI=0 , I;
    
    strncpy(tmp, str+0, 4);
    tmp[4] = '\0';
    sscanf(tmp, "%d", &H);
        
    strncpy(tmp, str+4, 4);
    tmp[4] = '\0';
    sscanf(tmp, "%d", &K);

    strncpy(tmp, str+8, 4);
    tmp[4] = '\0';
    sscanf(tmp, "%d", &L);

    strncpy(tmp, str+12, 8);
    tmp[8] = '\0';
    sscanf(tmp, "%f", &I);

    if(strlen(str)>20){  
        strncpy(tmp, str+20, 8);
        tmp[8] = '\0';
        sscanf(tmp, "%f", &sI);
    }
        
    *h=H; *k=K; *l=L; *a=I; *sa=sI;   
}

void shelx2mmcif(FILE *fout,  char *inpfile, char *type)
/*convert SHELX reflection file (3I4,2F8.2,I4 for SHELX ) to CIF format
*/
{
    char sf_type[10],  tmp[10], **line, free='o';
    int len, H, K, L, i,n, nflag=0, flag=0, nline, k1=0,k2=0;
    float sI , I;

    line=get_lines_from_file(inpfile, &nline);
    strcpy(sf_type,"F");
    for(i=0; i<nline; i++){ //test flag, I, format
        if(!strncmp(line[i], "   0   0   0", 12) || strstr_space(line[i], "H   K   L")) continue;
        
        rid_of_end_space(line[i]);
        
        len=strlen(line[i]);
        if(len==28){
            k1++;
        }else if (len==32){
            k2++;
        }
        
        strncpy(tmp, line[i] + 12, 8);
        tmp[8]='\0';
        if(atof(tmp)<0) {
            strcpy(sf_type,"I");
        }
        strncpy(tmp, line[i]+28, 4);
        tmp[4] = '\0';
        n=sscanf(tmp, "%d", &flag);
        if(n==1 && flag==-1) nflag++;
        
    }

    if((k1>0 && k1<nline-4) || (k2>0 && k2<nline-4)) {
        printf("WARNING! IT MAY NOT BE A SHELX FORMAT (%d %d %d)\n", k1,k2,nline);
    }

    
    if(!strcmp_case(type,"F") && !strcmp_case(sf_type,"I")){
        printf("Warning! Your given SF type (%s) is different from the detected type (%s)\n", type,sf_type);
        printf("Please check with the format.\n");
    }

    if(!strcmp_case(type,"F")){
        printf("Your SF type is given as F.\n");
        sf_data_header(fout, 1, 1);
    }else if(!strcmp_case(type,"I")  ){
        printf("Your SF type is given as I.\n");
        sf_data_header(fout, 2, 1);
    }else if(!strcmp_case(sf_type,"F")){
        printf("Your SF type is assumed to be F. If not, please give -sf_type ?\n");
        sf_data_header(fout, 1, 1);
    }else if(!strcmp_case(sf_type,"I") ){
        printf("Your SF type is assumed to be I. If not, please give -sf_type ? \n");
        sf_data_header(fout, 2, 1);
    }else{
        printf("Your SF type is assumed to be I. If not, please give -sf_type ? \n");
        sf_data_header(fout, 2, 1);
    }
    
    for(i=0; i<nline; i++){ 
        if(!strncmp(line[i], "   0   0   0", 12) || strstr_space(line[i], "H   K   L")) continue;
        
        free='o';
        formated_4_8(line[i], &H, &K, &L, &I, &sI);
        
        if(H==0 && K==0 && L==0) continue;
        if(H<-800 || H>800 || K<-800 || K>800 || L<-800 || L>800 ){
            printf("Warning! H K L indices > 800 (or <-800). (%s)\n", line[i]);
        }
        if(sI <0){
            printf("Warning! sigma <0 (%s)\n", line[i]);
        }
        
        strncpy(tmp, line[i]+28, 4);
        tmp[4] = '\0';
        sscanf(tmp, "%d", &flag);
        if(nflag>3 && flag==-1) free='f';
        if(nflag <=0 && flag==1) free='f';
            
        fprintf(fout,"1 1 1 %4d %4d %4d %9.2f %8.2f %c\n",  H, K, L, I, sI, free);
    }
    if(nline) free_memory_2d(line, nline);
}

void saint2mmcif(FILE *fout,  char *inpfile)
/*convert SAINT/XPREP reflection file  to CIF format
*/
{
    char  str[512];
    int  H, K, L, n=0;
    float sI=0 , I;
    FILE *fp;

    if((fp = fopen(inpfile, "r"))==NULL) open_file_error(inpfile, "saint2mmcif");
    
    n=0;
    sf_data_header(fout, 2, 0);
    while (fgets(str, sizeof str, fp) != NULL) {
        formated_4_8(str, &H, &K, &L, &I, &sI);
        fprintf(fout,"1 1 1 %4d %4d %4d %10.2f %10.2f\n",  H, K, L, I, sI);
        n++;
    }
    fclose(fp);
    if(n<10) {
        printf("Error! conversion to mmcif failed\n");
        return;
    }
}

void fi2mmcif(FILE *fout,  char *inpfile, char *type)
/*convert reflection file (3I4,2F8.2,2F8.2,I4 for SHELX ) to CIF format
here F and I exist also with status
*/
{
    char  str[512],  tmp[10];
    int  H, K, L,  n=0, flag=0;
    float sI , I, sF , F;
    
    FILE *fp;

    if((fp = fopen(inpfile, "r"))==NULL) {
        open_file_error(inpfile, "fi2mmcif");
    }
    
    sf_data_header(fout, 0, 1);
    
    n=0;
    rewind(fp);
    while (fgets(str, sizeof str, fp) != NULL) {
        n++;
        
        sI = 0;
        if(strlen(str)<20)continue;
        
        formated_4_8(str, &H, &K, &L, &F, &sF);
        
        strncpy(tmp, str+28, 8);
        tmp[8] = '\0';
        sscanf(tmp, "%f", &I);
        
        strncpy(tmp, str+36, 8);
        tmp[8] = '\0';
        sscanf(tmp, "%f", &sI);
        
        strncpy(tmp, str+44, 4);
        tmp[4] = '\0';
        sscanf(tmp, "%d", &flag);

        if(flag==1){
            fprintf(fout,"1 1 1 %4d %4d %4d %8.2f %6.2f %8.2f %7.2f f\n",  H, K, L, F, sF, I, sI);
        }else {
            fprintf(fout,"1 1 1 %4d %4d %4d %8.2f %6.2f %8.2f %7.2f o\n",  H, K, L, F, sF, I, sI);
        }      
    }
    fprintf(fout,"#END OF REFLECTIONS\n");
    

    fclose(fp);    
}

void tnt2mmcif(FILE *fout,  char *inpfile)
/*convert TNT reflection file (A4, 4I4,2F8.2, F8.2,F8.4 ) to CIF format
  HKL, H, K, L,  F, sigmaF, phase, FOM in each row
*/
{

    char str[256], tmp[256],status='o';
    int H, K, L, num;
    float  F, sF;
    FILE *fp;

    if((fp = fopen(inpfile, "r"))==NULL) {
        open_file_error(inpfile, "tnt2mmcif");
    }
    
    printf("\nThe reflection data type is assumed to be amplitude (F)!\n");
//    sf_data_header(fout, 1, 0);
    sf_data_header(fout, 1, 1);

    while (fgets(str, sizeof str, fp) != NULL){
        sF = 0;
        upperstr(str);
        if(strncmp(str, "HKL ",4))continue;
        if(strlen(str)<23)continue;
        strcpy(tmp, str+4);
        
        formated_4_8(tmp, &H, &K, &L, &F, &sF);
        fprintf(fout,"1 1 1 %4d %4d %4d %9.1f %7.1f ",H, K, L, F, sF);
        if(strlen(str)>36){
            sscanf(str+36, "%d", &num);
            if(num==1)status='f';
        }
        fprintf(fout, " %c\n", status);
        
            
    }

    fclose(fp);
    
}

void xtalview2mmcif(FILE *fout,  char *inpfile)
/* this is a .fin format only !!!
File contains: h k l f1 sigma(f1) f2 sigma(f2). The f1 and f2 can be
Bijvoet pairs, fp and fph, or any other pairable data. If one of the
two observations is missing, then f should equal 0.0
and sigma(f) should equal 9999.0    
*/
{
    float  F, sF, Fp, sFp;
    char str[256], data_type[10];
    int H, K, L, m=0;
    
    FILE *fp;

    if((fp = fopen(inpfile, "r"))==NULL) {
        open_file_error(inpfile, "xtalview2mmcif");
    }

    
    strcpy(data_type, "F");
    printf("\nThe input reflection data type is assumed to be amplitude (F)!\n");
    
    sf_data_header(fout, 4, 0);
    
    while (fgets(str, sizeof str, fp) != NULL){
        m=sscanf(str, "%d %d %d  %f %f %f %f \n", &H,&K, &L, &F,&sF, &Fp, &sFp);
        if(m!=7) continue;

        if(F<0.01 && sF>9998){
            fprintf(fout,"1 1 1 %4d %4d %4d  ?  ? %8.2f %6.2f \n",
                    H, K, L,  Fp, sFp);
        }else if(Fp<0.01 && sFp>9998){
            fprintf(fout,"1 1 1 %4d %4d %4d  %8.2f %6.2f ?  ? \n",
                    H, K, L, F, sF);
        }else{
            fprintf(fout,"1 1 1 %4d %4d %4d  %8.2f %6.2f %8.2f %6.2f \n",
                    H, K, L, F, sF,  Fp, sFp);
        }
        
    }
    fclose(fp);
}


void xgen2mmcif(FILE *fout,  char *inpfile, char *type)
/*convert X-gen reflection file  to CIF format
h, k, l, res, F, sig(F), F+, sig(F+), F-, sig(F-), iflag
( 3I4, 1X, F6.4, 6(1X, F8.2) 1X, I2 )   separated by a space
X-GEN and  XENGEN are the same.
*/
{
    float F=0, sF=0,  Fp=0, sFp=0,  Fm=0, sFm=0;
    char str[512];
    int H, K, L, n, m=0;
    FILE *fp;

    if((fp = fopen(inpfile, "r"))==NULL) {
        open_file_error(inpfile, "xgen2mmcif");
    }
    
    if(!strcmp(type,"I")){
        sf_data_header(fout, 3, 0);
        fprintf(fout, "_refln.intensity_meas\n" );
        fprintf(fout, "_refln.intensity_sigma\n" );
        printf("\nThe reflection data type is assumed to be amplitude (I)!\n");
    }else{
        sf_data_header(fout, 4, 0);
        fprintf(fout, "_refln.F_meas_au\n" );
        fprintf(fout, "_refln.F_meas_sigma_au \n" );
        printf("\nThe reflection data type is assumed to be amplitude (F)!\n");
    }
    n=0;
    while (fgets(str, sizeof str, fp) != NULL){
        if(strlen(str)<61)continue;
        m=sscanf(str, "%d %d %d %*s %f %f %f %f %f %f\n",
               &H,&K, &L,&F,&sF, &Fp, &sFp, &Fm, &sFm);
        if (m<9) continue;
        
        fprintf(fout,"1 1 1 %3d %3d %3d  %8.2f %6.2f %8.2f %6.2f  %8.2f %6.2f \n",
                H, K, L, Fp, sFp,  Fm, sFm, F, sF);
        n++;
        
    }
    fclose(fp);
    
}
void multan2mmcif(FILE *fout,  char *inpfile, char *type)
/*
The file has h, k, l, f, imt in FORMAT(3I4,7X,F7.0,I6),
where imt=0 for a good reflection.  
*/
{
    char str[256];
    int H, K, L, m=0;
    float F=0;
    FILE *fp;

    if((fp = fopen(inpfile, "r"))==NULL) {
        open_file_error(inpfile, "multan2mmcif");
    }
    if(!strcmp(type,"I")){
        sf_data_header(fout, 2, 0);
        printf("Input data type is intensity (I)!\n");
    }else{
        sf_data_header(fout, 1, 0);
        printf("Input data type is amplitude (F)!\n");
    }
    
     
    while (fgets(str, sizeof str, fp) != NULL){
        if(strlen(str)<30)continue;
        m=sscanf(str, "%d %d %d  %f \n", &H,&K, &L,&F);
        if (m<4) continue;
        fprintf(fout,"1 1 1 %3d %3d %3d  %8.2f ? \n", H, K, L, F);
        
    }
    fclose(fp);

}


void main2mmcif(FILE *fout,  char *inpfile)
/*convert main reflection file (INDEX,FOBS,SIGMA,TEST ) to CIF format
The default format "main", which assumes keyword based records
based on old X-plor version format.
INDEX    8     1   -20  FOBS  1262.150  SIGMA 344.3 PHA 132.23 WE 0.345 TEST 0
http://www-bmb.ijs.si/
*/
{
    char status='o', str[256], **line;
    float  F=0, sF=0, pha=0, we=0;
    int i, H=0, K=0, L=0, n=0,n1=0,n2=0, phase=0, test=0, nstr=0;
    FILE *fp;


    if((fp = fopen(inpfile, "r"))==NULL) {
        open_file_error(inpfile, "main2mmcif");
    }

    n=0;
    while (fgets(str, sizeof str, fp) != NULL){
        if(pattern5(str, "INDE", " FOBS ", ".", "", " TEST")){
            test=1;
        }else if(pattern5(str,"INDE", " FOBS ", ""," PHA ", " WE ")){
            phase=1;
        }
        if(pattern5(str,"INDE", " FOBS ", ".", "", "")  && n++>40) break;
    }
    
    (test)?sf_data_header(fout, 1, 1):sf_data_header(fout, 1, 0);
    if(phase){
        fprintf(fout, "_refln.phase_meas\n");        
        fprintf(fout, "_refln.fom\n");              
    }
            
    if (n<10){
        printf("Error! Too few reflection data.\n");
        fclose(fp);   
        return;
    }
    
    rewind(fp);
    while (fgets(str, sizeof str, fp) != NULL){
        
        sF = 0;
        nstr=0;
        upperstr(str);

        
        line = string_token(str, " " , &nstr);
        
        for(i=0; i<nstr; i++){
            H=0; K=0; L=0;
            if(strstr(line[i], "INDE")){
                H=atoi(line[i+1]);
                K=atoi(line[i+2]);
                L=atoi(line[i+3]);
                printf("sstri=%s %d %d %d \n", line[i], H, K, L);
                i=i+2;
                n1=1;
            }else if(strstr(line[i], "FOBS")){
                F=atof(line[i+1]);
                n2=1;
            }else if(strstr(line[i], "SIG")){
                sF=atof(line[i+1]);
            }else if(strstr(line[i], "PHA")){
                pha=atof(line[i+1]);
            }else if(strstr(line[i], "WE")){
                we=atof(line[i+1]);
            }else if(strstr(line[i], "TEST")){
                (atoi(line[i+1])>0)? status='f' : status='o';
            }
        }
        if(!(n1 && n2) || (H==0 && K==0 && L==0) ) continue;
        
        if(test ==1){
            fprintf(fout,"1 1 1 %4d %4d %4d %8.2f %7.2f %c", H, K, L,  F, sF, status);
        }else{
            fprintf(fout,"1 1 1 %4d %4d %4d %8.2f %7.2f ", H, K, L,  F, sF);
        }
        if(phase)fprintf(fout," %6.1f %6.3f", pha, we);
        fprintf(fout,"\n");
        if(nstr) free_memory_2d(line, nstr);
        
    }
    fclose(fp);   
}

void epmr2mmcif(FILE *fout,  char *inpfile,char *type)
/*convert EPMR reflection file (free format ) to CIF format
   H, K, L,  Fo,  in each row
*/
{

    float  F, sF=0, status;
    char  str[512], ch;
    int H, K, L, n, nref=0;
    FILE *fp;


    if((fp = fopen(inpfile, "r"))==NULL) {
        open_file_error(inpfile, "epmr2mmcif");
    }

                   
    while (fgets(str, sizeof str, fp) != NULL){
        
        if(strlen(str)<20)continue;
        n=sscanf(str,"%d%d%d%f%f%f", &H, &K, &L, &F, &sF ,&status);
        
        if(n==4 || n==5 || n==6 )
            nref++;
        
        if(nref==1 && (n==4 || n==5)){
            if(!strcmp(type, "F")){
                
                printf("\nThe reflection data type is  amplitude (F)!\n");
                sf_data_header(fout, 1, 0);
            }else if(!strcmp(type, "I")){
                
                printf("\nThe reflection data type is intensity (I)!\n");
                sf_data_header(fout, 2, 0);
            }else
                sf_data_header(fout, 1, 0);
        }else if(nref==1 &&  n==6){
            if(!strcmp(type, "F")){
                printf("\nThe reflection data type is amplitude (F)!\n");
                sf_data_header(fout, 1, 1);
            }else if(!strcmp(type, "I")){
                printf("\nThe reflection data type is intensity (I)!\n");
                
                sf_data_header(fout, 2, 1);
            }else
                sf_data_header(fout, 1, 1);
        }
        
        
        if(n==4){
            fprintf(fout,"1 1 1 %4d %4d %4d %10.2f 0.0 \n", H, K, L, F );
        }else if (n==5){
            fprintf(fout,"1 1 1 %4d %4d %4d %10.2f %8.2f\n", H, K, L, F, sF );
        }else if (n==6){
            if(status<1)
                ch='f';
            else
                ch='o';
                
            fprintf(fout,"1 1 1 %4d %4d %4d %10.2f %8.2f %c\n", H, K, L, F, sF, ch );
        }else
            continue;
    }

    fclose(fp);
}

void cryst_infor(FILE *fout, float *a, char *sym,int num)
{
    char symmetry[20];
    
    fprintf(fout, "#\n");
    
    if(num>0)fprintf(fout, "_symmetry.Int_Tables_number  %d\n", num);
    if(strlen(sym)>1){        
        space_group_convertor(sym, symmetry); 
        fprintf(fout, "_symmetry.space_group_name_H-M  '%s'\n", symmetry);
    }
    if(a[0]>0.5 && a[2]>0.5){
        fprintf(fout, "#\n");
        fprintf(fout, "_cell.length_a  %.3f\n", a[0]);
        fprintf(fout, "_cell.length_b  %.3f\n", a[1]);
        fprintf(fout, "_cell.length_c  %.3f\n", a[2]);
        fprintf(fout, "_cell.angle_alpha  %.3f\n", a[3]);
        fprintf(fout, "_cell.angle_beta  %.3f\n", a[4]);
        fprintf(fout, "_cell.angle_gamma  %.3f\n", a[5]);
    }
    
}


void hkl2mmcif(FILE *fout, char *filename)
/* get the right format for HKL2000/scalepack
    
1, NO ANOMALOUS flag ----- h, k, l, I, sigma(I)  (3I4, 2F8.1) length=28
2, WITH ANOMALOUS flag --- h, k, l, I+, sigma(I+), I-, sigma(I-) ;
   (3I4, 4F8.1) length=44
3, NO MERGE  --- h, k, l, asymmetric unit #, I, sigma(I) ; (4I4,2F8.1)  
   ANOMALOUS flag has no effect.  length=32
4, NO MERGE original index --- original hkl, unique hkl, batch (film)
   number, a flag (0 = centric, 1 = I+, 2 = I-), another flag (0 =
   hkl reflecting above the spindle, 1 = hkl reflecting below the spindle), 
   the asymmetric unit of the reflection, I (scaled, Lorentz and Polarization
   corrected), and the sigma of I. The format is (6i4, i6, 2i2, i3, 2f8.1).
   (RAW data!!)  length=53

*/
    
{
    char str[512],symmetry_in[20],symmetry[20];
    int H, K, L, len=0, n, ncell=0;
    int n1=0,n2=0,n3=0,n4=0,nref=0;
    float a[10], ii=0, sii=0, I, sI ;
    FILE  *finp;
    
    if((finp = fopen(filename, "r"))==NULL) open_file_error(filename, "hkl2mmcif");

    nref=0;
    while (fgets(str, sizeof str, finp) != NULL) {
        rid_of_end_space(str);
        len=strlen(str);
        if(len>60 && pattern5(str,".",".", ".",".","")){
            n=sscanf(str,"%f%f%f%f%f%f",
                     &a[0],&a[1],&a[2],&a[3],&a[4],&a[5]);
        
            if(n==6 && a[0]>1.0 && a[2]>1.0 && ncell==0){
                get_string(str, 60, 70, symmetry_in);
                space_group_convertor(symmetry_in,symmetry);
                cryst_infor(fout, a,symmetry,-1);
                ncell++;
                
            }
        }else if(len==28 || len==29){ //I
            n1++;
        }else if(len==44 || len==45){ //I+, I-
            n2++;
        }else if(len==35 || len==36){
            n3++;
        }else if(len==53 || len==54){
            n4++;
        }
        if(nref++>3000)break;
    }
    
    printf("The input reflection data type is assumed to be intensity (I)!\n");
    (n2>10)?sf_data_header(fout,3, 0) : sf_data_header(fout,2, 0);
    
    nref=0;
    rewind(finp);
    while (fgets(str, sizeof str, finp) != NULL) {
        len=strlen(str);
        if(!(len==28 || len==29 || len==44 || len==45 ||
             len==35 || len==36 || len==53 || len==54))continue;
        
        nref++;
        
        if(n2>10){ //I+
            formated_4_8(str, &H, &K, &L, &I, &sI);
            if(len==28 || len==29)
                fprintf(fout,"1 1 1 %4d %4d %4d %9.1f %7.1f        0.0    -1.0\n",
                        H, K, L, I, sI);
            else{
                ii=get_float_by_length(str, 28, 35);
                sii=get_float_by_length(str, 36, 43);
                fprintf(fout,"1 1 1 %4d %4d %4d %9.1f %7.1f  %9.1f %7.1f\n",
                        H, K, L, I, sI, ii, sii);
            }
            
        }else if(n3>10){//NO MERGE
            H=get_int_by_length(str, 0, 3);
            K=get_int_by_length(str, 4, 7);
            L=get_int_by_length(str, 8, 11);
            
            I=get_float_by_length(str, 19, 26);
            sI=get_float_by_length(str, 27, 34);
            
            fprintf(fout,"1 1 1 %4d %4d %4d  %9.1f %7.1f\n", H, K, L,  I, sI);
            
        }else if(n4>10){//NO MERGE orin
            if(len<52) continue;
            
            H=get_int_by_length(str, 0, 3);
            K=get_int_by_length(str, 4, 7);
            L=get_int_by_length(str, 8, 11);
            
            I=get_float_by_length(str, 37, 44);
            sI=get_float_by_length(str, 45, 52);

            fprintf(fout,"1 1 1 %4d %4d %4d  %9.1f %7.1f\n", H, K, L,  I, sI);
            
        }else {
            formated_4_8(str, &H, &K, &L, &I, &sI);
            fprintf(fout,"1 1 1 %4d %4d %4d  %9.1f %7.1f\n", H, K, L,  I, sI);
        }
        
    }
    fclose(finp);
    
    if(!(n1<10 || n2<10  || n3<10  || n4<10 || nref<10) ){
        printf("Error! The HKL2000/SCALEPACK format is wrong!\n");
    }
}


void dtrek2mmcif(FILE *fout, char *filename)
/* write reflections from d*trek */
{
    char str[512];
    int h=0, k=0, l=0, m, n1=0, sg_num=0, nref=0;
    float i, sigmai, plus_i, plus_s, minus_i, minus_s, a[10];
    FILE *finp;

    if((finp = fopen(filename, "r"))==NULL) {
        open_file_error(filename, "dtrek2mmcif");
    }

    nref=0;
    while (fgets(str, sizeof str, finp) != NULL) {
        if(nref++>500) break;
        if(pattern5(str,"CRYSTAL_SPACEGROUP=", ";", "", "","")){
            sg_num=get_int(str, "=", ";");
        }else if(pattern5(str,"CRYSTAL_UNIT_CELL=", ";", "", "","")){
            sscanf(get_substr(str, "=", ";"), "%f %f %f %f %f %f",
                   &a[0],&a[1],&a[2],&a[3],&a[4],&a[5]);
        }        
        m=sscanf(str, " %d%d%d %f%f %f%f %f%f",
                 &h, &k, &l, &i, &sigmai, &plus_i, &plus_s, &minus_i, &minus_s);
        if(m==9)n1++;
            
    }
    if(a[0]>0 && a[1]>0 && a[5]>0) cryst_infor(fout, a,(char *)"?",sg_num);
        
    printf("The reflection data type is assumed to be intensity (I)!\n");
    if(n1>50){
        sf_data_header(fout, 3, 0);
        fprintf(fout, "_refln.intensity_meas\n" );
        fprintf(fout, "_refln.intensity_sigma\n" );
    }else
        sf_data_header(fout, 2, 0);

    rewind(finp);
    while (fgets(str, sizeof str, finp) != NULL) {
        m=sscanf(str, " %d%d%d %f%f %f%f %f%f",
                 &h, &k, &l, &i, &sigmai, &plus_i, &plus_s, &minus_i, &minus_s);
        
        if(m==5 && sigmai >=0){
            fprintf(fout,"1 1 1 %4d %4d %4d %9.2f %7.2f\n", h, k, l, i, sigmai);
        }else if(m==9){
            fprintf(fout,"1 1 1 %4d %4d %4d %9.2f %7.2f %9.2f %7.2f %9.2f %7.2f\n",
                    h, k, l, plus_i, plus_s, minus_i, minus_s, i, sigmai);
        }
    }
    
    fclose(finp);
    
}

void xscale2mmcif(FILE *fout, char *filename)
// XSCALE || XDS
{
    char **line;
    int i,h,k,l, nline,start=0, npair=0, sg_num=0;
    float a1,a2,a3,a4, b[6];
    
    line=get_lines_from_file(filename, &nline);
    
    for(i=0; i<nline-10; i++){
        if(pattern5(line[i],"!SPACE_GROUP_NUMBER=", "", "", "","")){
            sg_num=get_int_after_id(line[i], "=");
        }else if(pattern5(line[i],"!UNIT_CELL_CONSTANTS=", ".", ".", ".","")){
            sscanf(strstr(line[i], "=")+1, "%f %f %f %f %f %f",
                   &b[0],&b[1],&b[2],&b[3],&b[4],&b[5]);
        }
        
        if(strstr(line[i],"!END_OF_HEADER")) {
            if(b[0]>0 && b[1]>0 && b[5]>0) 
                cryst_infor(fout, b,(char *)"?",sg_num);
            
            start=i+1;
            break;
        }
        
        if(strstr_case(line[i], "!ITEM_H=1") &&
           strstr_case(line[i+1], "!ITEM_K=2") &&
           strstr_case(line[i+2], "!ITEM_L=3") &&
           strstr_case(line[i+3], "!ITEM_IOBS=4") &&
           strstr_case(line[i+4], "!ITEM_SIGMA(IOBS)=5") &&
           strstr_case(line[i+5], "!ITEM_IOBS'=6") &&
           strstr_case(line[i+6], "!ITEM_SIGMA(IOBS')=7"))
           npair=2;
    }

    if(npair==2){
        sf_data_header(fout, 3, 0);
        for(i=start; i<nline; i++){
            sscanf(line[i], "%d%d%d%f%f%f%f\n", &h,&k,&l, &a1, &a2, &a3, &a4);
            fprintf(fout,"1 1 1 %3d %3d %3d %9.2f %7.2f %9.2f %7.2f\n", h,k,l, a1, a2,a3,a4);
        }
    }else{
        sf_data_header(fout, 2, 0);
        for(i=start; i<nline; i++){
            if(npair==0){
                sscanf(line[i], "%d%d%d%f%f\n", &h,&k,&l, &a1, &a2);
                fprintf(fout,"1 1 1 %3d %3d %3d %9.2f %7.2f \n", h,k,l, a1, a2);
            }
        }
    }
     
    if(line && nline) free_memory_2d(line, nline);
}


void  cif2mmcif(FILE *fout, char *inpfile) 
// convert CIF (shelx) to mmCIF reflection items.
{
    char str[512], tmp[512], **line=NULL, **item=NULL, **lines=NULL ;
    int  index[80], i, j,n, nt, nitem_tot=0;
    int *line_index, nstr=0, nitem=0, nline=0,nline_tmp=0, start_hkl=0;
    float cell[6];
    FILE   *finp;

    
    if((finp=fopen(inpfile, "r")) == NULL){
        open_file_error(inpfile,"cif2mmcif");
    }

    for(i=0; i<80; i++){
        index[i]=-10;
    }
    
   
    nline_tmp=0;
    while(fgets(str, sizeof str, finp) != NULL){ //get # of lines 
        nline_tmp++;
    }

    lines = new char *[nline_tmp];
    line_index = new int[nline_tmp];
   
    rewind(finp);
    nline=0;
    nt=0;
    while(fgets(str, sizeof str, finp) != NULL){
        n=sscanf(str, "%s", tmp );
        if(n<=0) continue;
        
        
        if(!strcmp_case(tmp,"_cell_length_a")){
            cell[0] =get_float_after_id(str, "length_a");
        }else if(!strcmp_case(tmp,"_cell_length_b")){
            cell[1] =get_float_after_id(str, "length_b");
        }else if(!strcmp_case(tmp,"_cell_length_c")){
            cell[2]=get_float_after_id(str, "length_c");
        }else if(!strcmp_case(tmp,"_cell_angle_alpha")){
            cell[3]=get_float_after_id(str, "angle_alpha");
        }else if(!strcmp_case(tmp,"_cell_angle_beta")){
            cell[4]=get_float_after_id(str, "angle_beta");
        }else if(!strcmp_case(tmp,"_cell_angle_gamma")){
            cell[5]=get_float_after_id(str, "angle_gamma");
        }

        rid_of_front_end_space(str);
        nt++;
        
        if(strlen(str)<1 || str[0]=='#') continue;
        if(nline>10 && !strncmp(str,"data_", 5))break; //next block
        lines[nline] = new char [strlen(str)+1];
        strcpy(lines[nline],str);
        line_index[nline] = nt;
        nline++;
    }
//    printf("nline, nline_tmp= %s %d %d\n", inpfile, nline,nline_tmp);

    cryst_infor(fout, cell,(char *)"?" , 0);
    

    nitem=0;
    for(i=0; i<nline-5; i++){  //get cif items
        if(!strncmp(lines[i+0],"_refln_",7) &&
           !strncmp(lines[i+1],"_refln_",7) &&
           !strncmp(lines[i+2],"_refln_",7) &&
           !strncmp(lines[i+3],"_refln_",7)){
            
            nitem=0;
            item = new char *[80];
            for(j=i; j<nline-5 ; j++){
                if(strncmp(lines[j], "_refln_", 7)) {
                    start_hkl=j;
                    break;
                }
                item[nitem] = new char [strlen(lines[j])+1];
                sscanf(lines[j], "%s", item[nitem]);
                nitem++;
            }
            if(strncmp(lines[j], "_refln_", 7)) break; 
        }
    }
    
    if(nitem<=0) {
        printf("The input file (%s) has no correct tokens. \n", inpfile);
        printf("Please make sure it is a CIF file for small molecule.\n");
        if(lines && nline>0) free_memory_2d(lines, nline);
        fclose(finp);
        return;
    }
    
    get_cif_item_index(nitem, item, index, &nitem_tot);
    
    n=0;
    for(i=start_hkl; i<nline; i++){
        strcpy(tmp,lines[i]);
        line = string_token(tmp, " " , &nstr);
        
        if(nstr != nitem ){
            printf("Error at line = %d\n", line_index[i]);
            continue;
        }
        n++;
        write_new_mmcif(fout, n, item, index, line, nitem_tot);
        if(nstr>0) free_memory_2d(line, nstr);
    }
    if(item && nitem) free_memory_2d(item, nitem);
    if(lines && nline) free_memory_2d(lines, nline);
    delete [] line_index;
    
    
    fclose(finp);
}

void get_cif_item_index(int n, char **item, int index[], int *nitem_tot)  
//correspondance of total number of CIF and mmCIF items.
{
    int i;
    
    for(i=0; i<n; i++){
        if(!strcmp_case(item[i], "_refln_index_h")){
            strcpy(item[i],  "_refln.index_h");
            index[0]=i;
        }else if(!strcmp_case(item[i], "_refln_index_k")){
            strcpy(item[i],  "_refln.index_k");
            index[1]=i;
        }else if(!strcmp_case(item[i], "_refln_index_l")){
            strcpy(item[i],  "_refln.index_l");
            index[2]=i;
        }else if(!strcmp_case(item[i], "_refln_F_meas")){
            strcpy(item[i],  "_refln.F_meas_au");
            index[3]=i;
        }else if(!strcmp_case(item[i], "_refln_F_sigma")){
            strcpy(item[i],  "_refln.F_meas_sigma_au");
            index[4]=i;
        }else if(!strcmp_case(item[i], "_refln_F_squared_meas")){
            strcpy(item[i],  "_refln.intensity_meas");
            
            index[5]=i;
        }else if(!strcmp_case(item[i], "_refln_F_squared_sigma")){
            strcpy(item[i],  "_refln.intensity_sigma");
            index[6]=i;

        }else if(!strcmp_case(item[i], "_refln_F_calc")){
            strcpy(item[i],  "_refln.F_calc_au");
            index[7]=i;
        }else if(!strcmp_case(item[i], "_refln_F_squared_calc")){
            strcpy(item[i],  "_refln.F_squared_calc");
            index[8]=i;
            
        }else if(!strcmp_case(item[i], "_refln_observed_status")){
            strcpy(item[i],  "_refln.status");
            index[9]=i;

        }else if(!strcmp_case(item[i], "_refln_intensity_meas")){
            strcpy(item[i],  "_refln.intensity_meas");
            index[10]=i;
        }else if(!strcmp_case(item[i], "_refln_intensity_sigma")){
            strcpy(item[i],  "_refln.intensity_sigma");
            index[11]=i;
            
        }else if(!strcmp_case(item[i], "_refln_fom")){
            strcpy(item[i],  "_refln.fom");
            index[12]=i;
            
        }else if(!strcmp_case(item[i], "_refln_phase_calc")){
            strcpy(item[i],  "_refln.phase_calc");
            index[13]=i;
            
        }else if(!strcmp_case(item[i], "_refln_status")){
            strcpy(item[i],  "_refln.status");
            index[14]=i;
        }
        
    }
    *nitem_tot=15;
}

void write_new_mmcif(FILE *fout, int n, char **item, int *index, char **line,
                     int nitem_tot)
{
    int i, k=-1;
    
    if(n==1){ // write the possible items
        fprintf(fout,"\n#\nloop_\n");
        fprintf(fout, "_refln.crystal_id \n");
        fprintf(fout, "_refln.wavelength_id \n");
        fprintf(fout, "_refln.scale_group_code\n");
        
        for(i=0; i<nitem_tot; i++){
            if(index[i] >=0){
                fprintf(fout,"%s\n", item[index[i]]);
            }
            
        }
    }

    for(i=0; i<nitem_tot; i++){
        if(index[i] >=0){
            if(strstr(item[index[i]], "_refln.status")){
                    //printf("index for status=%d\n", i);
                k=i;
            }
        }
    }

    
    for(i=0; i<nitem_tot; i++){// write the possible values
        if(i<=2 && i>=0){
            if(i==0)
                fprintf(fout,"1 1 1 ");
            if(index[i] >=0)
                fprintf(fout," %4s", line[index[i]]);
        }else if(i==k){ //for status
            if (line[index[i]][0] == 'o' || line[index[i]][0] == 'f' ){
                fprintf(fout," %c ", line[index[i]][0]);               
            }else{
                if(atof(line[index[i]])>0.5)
                    fprintf(fout," f ");               
                else
                    fprintf(fout," o ");   
            }
            
                
        }else{
            
            if(index[i] >=0)
                fprintf(fout," %10s", line[index[i]]);
        }
                
//        if(index[i] >=0)fprintf(fout," %s", line[index[i]]);
    }
    
    fprintf(fout,"\n");
    
}

 
void sf_data_header(FILE *fout, int I_F_type, int status )
// status>0 Rfree exist
// I_F_type=0, F & I ; =1, F only; =2, I only; =3, I(+)(-) only; =4, F(+)(-) only;
{
    fprintf(fout, "#\n");

    fprintf(fout, "loop_\n");
    fprintf(fout, "_refln.crystal_id \n");
    fprintf(fout, "_refln.wavelength_id \n");
    fprintf(fout, "_refln.scale_group_code\n");
    fprintf(fout, "_refln.index_h\n");
    fprintf(fout, "_refln.index_k\n");
    fprintf(fout, "_refln.index_l\n");
    
    
    if(I_F_type ==0 ){
        fprintf(fout, "_refln.F_meas_au\n" );
        fprintf(fout, "_refln.F_meas_sigma_au \n" );
        fprintf(fout, "_refln.intensity_meas\n" );
        fprintf(fout, "_refln.intensity_sigma\n" );
        
    }else if(I_F_type ==1){  
        fprintf(fout, "_refln.F_meas_au\n" );
        fprintf(fout, "_refln.F_meas_sigma_au \n" );
        
    }else if(I_F_type ==2){  
        fprintf(fout, "_refln.intensity_meas\n" );
        fprintf(fout, "_refln.intensity_sigma\n" );
        
    }else if(I_F_type ==3){  
        fprintf(fout, "_refln.pdbx_I_plus\n" );
        fprintf(fout, "_refln.pdbx_I_plus_sigma\n" );
        fprintf(fout, "_refln.pdbx_I_minus\n" );
        fprintf(fout, "_refln.pdbx_I_minus_sigma\n" );
        
    }else if(I_F_type ==4){  
        fprintf(fout, "_refln.pdbx_F_plus\n" );
        fprintf(fout, "_refln.pdbx_F_plus_sigma\n" );
        fprintf(fout, "_refln.pdbx_F_minus\n" );
        fprintf(fout, "_refln.pdbx_F_minus_sigma\n" );
        
    } else{
        printf("Data-type is wrong! (It should be either (F) or (I))\n");
        exit(0);
    }
    if(status >0 )fprintf(fout, "_refln.status\n" );

}



/////////////////////////// Unix convert MTZ to mmcif /////////////////////
/*
 mmCIF                            Data_type  MTZ labels   
                                              (possible)   
 _refln.wavelength_id                 .           .              Always written
 _refln.crystal_id                    .           .              Always written
 _refln.scale_group_code              .           .              Always written
 _refln.index_h                       H           K              Always written
 _refln.index_k                       H           K              Always written
 _refln.index_l                       H           L              Always written
 _refln.status            	      I           FREE           Always written
                                                                                           
 _refln.F_meas_au         	      F           FP, F_X2    
 _refln.F_meas_sigma_au               Q           SIGFP, SIGF_X2  
 _refln.F_calc            	      F           FC     
 _refln.phase_calc        	      P           PHIC   
 _refln.phase_meas        	      P           PHIB   
 _refln.fom               	      W           FOM    
 _refln.intensity_meas    	      J           I      
 _refln.intensity_sigma   	      Q           SIGI   

_refln.pdbx_FWT                       F
_refln.pdbx_PHWT                      P
_refln.pdbx_DELFWT                    F
_refln.pdbx_DELPHWT                   P

 
 _refln.pdbx_HL_A_iso                 A       	HLA    
 _refln.pdbx_HL_B_iso                 A       	HLB    
 _refln.pdbx_HL_C_iso                 A       	HLC    
 _refln.pdbx_HL_D_iso                 A       	HLD    
 _refln.pdbx_F_plus         	      G           F(+)   
 _refln.pdbx_F_plus_sigma   	      L           SIGF(+)
 _refln.pdbx_F_minus        	      G           F(-)   
 _refln.pdbx_F_minus_sigma  	      L           SIGF(-)
 _refln.pdbx_anom_difference          D           DP   
 _refln.pdbx_anom_difference_sigma    Q           SIGDP
 _refln.pdbx_I_plus                   K           I(+)   
 _refln.pdbx_I_plus_sigma             M           SIGI(+)
 _refln.pdbx_I_minus                  K           I(-)   
 _refln.pdbx_I_minus_sigma            M           SIGI(-)      

*/

 

void get_type_label_sets(char *inpfile, char *label,char *type,  char *sets)
//get data sets, labels, types
{
    char  str[2000], log[256];
    
    FILE *flog=NULL;
    
    sprintf(log, "mtzdmp.log"); //get log file only
    delete_file(log);

    mtzdmp_file(inpfile, "/dev/null", 10); //testing
    mtzdmp_file(inpfile, log, 10);
    
    if((flog=fopen(log, "r"))==NULL){
        printf("Error! can not open the LOGfile (%s)\n",log);
        exit(0);
    }

    while(fgets(str, sizeof str, flog)!=NULL){ 
      if ( strstr(str,"* Associated datasets :")){
            while(fgets(str, sizeof str, flog)!=NULL){
                
                rid_of_front_end_space(str);
                if(strlen(str)>2){
                    strcpy(sets,str);
                    break;
                }
            }
        }else if ( strstr(str,"* Column Labels :")){
            while(fgets(str, sizeof str, flog)!=NULL){
                
                rid_of_front_end_space(str);
                if(strlen(str)>2){
                    strcpy(label,str);
                    break;
                }
            }
        }else if( strstr(str,"* Column Types :")){
            while(fgets(str, sizeof str, flog)!=NULL){
                rid_of_front_end_space(str);
                if(strlen(str)>2){
                    strcpy(type,str);
                    break;
                }
            }
        }        
    }
    fclose(flog);
    
}

int first_set_num(char **label_ptr,char **type_ptr,char **sets_ptr,int ntype)
// only test F and I for the data set (return the data set number).
{
    int j, n=0;
    
    for(j=3; j<ntype-1; j++){
        
        if      (type_ptr[j][0] == 'F' && type_ptr[j+1][0] == 'Q' ){
            n=atoi(sets_ptr[j]);
            break;
        }else if(type_ptr[j][0] == 'J' && type_ptr[j+1][0] == 'Q' ) {
            n=atoi(sets_ptr[j]);
            break;
        }else if(type_ptr[j][0] == 'K' && type_ptr[j+1][0] == 'M' ) {
            n=atoi(sets_ptr[j]);
            break;
        }else if(type_ptr[j][0] == 'G' && type_ptr[j+1][0] == 'L'  ) {
            n=atoi(sets_ptr[j]);
            break;
        }
    }
    return n;
}

void add_labels(FILE *finp, char **label_ptr, char **type_ptr,
                char **sets_ptr,int ntype, int d_set, int id)
// add labels to the script.
{
    
    int  j;
    
    fprintf(finp,"LABIN  ");  

    f_sig_pair(label_ptr, type_ptr,sets_ptr,  ntype, //FP, sigma
               'F','Q','F', d_set, "FP", "SIGFP",finp);
    
    f_sig_pair(label_ptr, type_ptr,sets_ptr,  ntype, //intensity, sigma
               'J','Q','I', d_set, "I", "SIGI",finp);
    
    f_sig_pair(label_ptr, type_ptr,sets_ptr,  ntype, //Ano_diff, sigma
               'D','Q', 'D', d_set, "DP", "SIGDP",finp);
    

    
    f_sig_pair(label_ptr, type_ptr,sets_ptr,  ntype, // F(+), sigma 
               'G', 'L',  'F',  d_set, "F(+)", "SIGF(+)",finp);
    
    f_sig_pair(label_ptr, type_ptr,sets_ptr,  ntype, // F(-), sigma 
               'G', 'L',  'F',  d_set, "F(-)", "SIGF(-)",finp);

              
    f_sig_pair(label_ptr, type_ptr,sets_ptr,  ntype, // I(+),sigma  
               'K', 'M',  'I',  d_set, "I(+)", "SIGI(+)",finp);
        
    f_sig_pair(label_ptr, type_ptr,sets_ptr,  ntype, // I(-),sigma  
               'K', 'M',  'I',  d_set, "I(-)", "SIGI(-)",finp);



    for(j=3; j<ntype; j++){

        if(atoi(sets_ptr[j])!= d_set) continue; //only for this set
        
        if(type_ptr[j][0] == 'P' ){ //PHIC, PHIB
            if(!strcmp_case(label_ptr[j], "PHIC") || !strcmp_case(label_ptr[j], "AC")){
                        
                fprintf(finp,"PHIC=%s  ",label_ptr[j] );
                
            }else if (strstr_case(label_ptr[j], "PHIB") ||
                      strstr_case(label_ptr[j], "PHIM")){
                fprintf(finp,"PHIB=%s  ",label_ptr[j] );
            }
        }else if(type_ptr[j][0] == 'A' ){ // HLA,HLB,HLC,HLD
            if(strstr_case(label_ptr[j], "HLA") ){
                fprintf(finp,"HLA=%s  ",label_ptr[j] );
            }else if(strstr_case(label_ptr[j], "HLB") ){
                fprintf(finp,"HLB=%s  ",label_ptr[j] );
            }else if(strstr_case(label_ptr[j], "HLC") ){
                fprintf(finp,"HLC=%s  ",label_ptr[j] );
            }else if(strstr_case(label_ptr[j], "HLD") ){
                fprintf(finp,"HLD=%s  ",label_ptr[j] );
            }
                    
        }else if(type_ptr[j][0] == 'W' ){ //FOM
            if(strstr_case(label_ptr[j], "FOM") ){
                fprintf(finp,"FOM=%s  ",label_ptr[j] );
            }
        }
    }


    
    if (id==2) { // for phasing sets
        fprintf(finp,"\n");
        return;
    }
    
    
    for(j=3; j<ntype-1; j++){
        if(atoi(sets_ptr[j])!= d_set) continue; //only for this set
        if(type_ptr[j][0] == 'F' && type_ptr[j+1][0] != 'Q'){ //FC
            if(!strcmp_case(label_ptr[j], "FC") || !strcmp_case(label_ptr[j], "Fcal")){
                fprintf(finp,"FC=%s  ",label_ptr[j] );
                break;  
            }
        }
    }
    
    
    for(j=3; j<ntype; j++){
        if(type_ptr[j][0] == 'I' ){ //status
            if(strstr_case(label_ptr[j], "free") ||
               strstr_case(label_ptr[j], "flag") ||
               strstr_case(label_ptr[j], "TEST")){
                fprintf(finp,"FREE=%s ",label_ptr[j] );
                break;
            } 
        }
    }
    
    
    fprintf(finp,"\n");
}

void mtz2mmcif_native(char *filename, char *outfile, char **label_ptr,
                      char **type_ptr,char **sets_ptr,int ntype, int nset) 
// using mtz2various to get the native data set.
{
    
    char script[512], str[512], command[256] ;
    int pid=0, n;
    FILE *fp=NULL, *finp=NULL ;

    printf("Converting mtz->mmcif for native data set=%d\n", nset);

    pid=getpid();
    sprintf(script, "%smtz2various_1.csh",PATHTMP);
    
    if((fp=fopen(script, "w"))==NULL){
        printf("Error! file (%s) has no written permission in directory (%s)!\n",
               script, PATHTMP);
        exit(0);
    }
    
    fprintf(fp,"#!/bin/csh \n\n");
    fprintf(fp,"# If you want the log file, replace /dev/null to test.log\n");

/*     
    fprintf(fp,"%s/bin/mtz2various HKLIN %s HKLOUT %s << EOF> /dev/null\n",
            CCP4, filename,outfile);
    fprintf(fp,"OUTPUT CIF data_mtz2cif \n");
    add_labels(fp, label_ptr, type_ptr, sets_ptr, ntype, nset, 1);
*/

    fprintf(fp,"%s/bin/mtz2cif HKLIN %s HKLOUT %s << EOF> /dev/null\n",
            CCP4, filename,outfile);
    fprintf(fp,"datablock data_mtz2cif \n");
    add_labels(fp, label_ptr, type_ptr, sets_ptr, ntype, nset, 1);
    
    if (strlen(FREERV)>0) fprintf(fp,"FREER  %s\n", FREERV);
    fprintf(fp,"END\n");
    fprintf(fp,"EOF\n");
    fprintf(fp,"#\n");

    fclose(fp);

    n=check_mtz_label(script);
    if(n==0) {
        printf("Error! No labels were written for MTZ to mmCIF conversion\n");
        exit(0);
    }
    
    sprintf(command, "/bin/csh -c \"source %s/include/ccp4.setup;chmod +x %s ; %s\" >/dev/null",CCP4,script, script);
    system(command);


    n=0;
    finp=fopen(outfile, "r");
    while(fgets(str, sizeof str, finp) != NULL){
        if (strstr(str,"_refln.")){
            while(fgets(str, sizeof str, finp) != NULL){
                if( str[0]=='#' || str[1]=='#') break;
                if (strchr(str, '*')){
                    n=1;
                    printf("Warning! ** in the data (%s).\n",str);
                    break;
                }
            }
        }
    }
    if (n){
        sprintf(command, "phenix.reflection_file_converter %s --cns=CNS_TMPP >/dev/null", filename);
        system(command);
        various_2_cif("CNS_TMPP.cns", outfile, "cns", "????", "");
        
    }
    
    

//    delete_file(script);

}


void get_mtzdmp_label(int i, char **label_ptr, char **type_ptr,
                      char **sets_ptr, int ntype, int nset, char *label)
// return a label for a cif token

{
    int nn=0;

    nn=atoi(sets_ptr[i]);
    strcpy(label, label_ptr[i]);

    
    if (      type_ptr[i][0] == 'H'  && !strcmp(label_ptr[i], "H")){
        strcpy(label,"index_h");
        
    }else if (type_ptr[i][0] == 'H'  && !strcmp(label_ptr[i], "K")){
        strcpy(label,"index_k");
        
    }else if (type_ptr[i][0] == 'H'  && !strcmp(label_ptr[i], "L")){
        strcpy(label,"index_l");

    }else if (nset==nn && type_ptr[i][0] == 'F' ) { //F
        if (!strcmp_case(label_ptr[i], "2FOFCWT") || !strcmp_case(label_ptr[i], "FWT")){
            strcpy(label,"pdbx_FWT");
            
        }else if (!strcmp_case(label_ptr[i], "FOFCWT") || !strcmp_case(label_ptr[i], "DELFWT")){
            strcpy(label,"pdbx_DELFWT");
            
        }else if (!strcmp_case(label_ptr[i], "FP") ||
                  (i<ntype-1 and type_ptr[i+1][0] == 'Q')){
            strcpy(label,"F_meas_au");
            
        }else if (!strcmp_case(label_ptr[i], "FC") ||
                  !strcmp_case(label_ptr[i], "Fcal")){ 
            strcpy(label,"F_calc");
        }
        
    }else if (nset==nn && type_ptr[i][0] == 'Q' ) {
        if (i>0 and type_ptr[i-1][0] == 'F'){
            strcpy(label,"F_meas_sigma_au");
        }else if (i>0 and type_ptr[i-1][0] == 'J'){
            strcpy(label,"intensity_sigma");
        }else if (i>0 and type_ptr[i-1][0] == 'D'){
            strcpy(label,"pdbx_anom_difference_sigma");
        }
        
        
    }else if (nset==nn && type_ptr[i][0] == 'J' ) {
        strcpy(label,"intensity_meas");
        
    }else if (nset==nn && type_ptr[i][0] == 'G' ) {
        if (strstr(label_ptr[i], "(+)")){
            strcpy(label,"pdbx_F_plus");
        }else if (strstr(label_ptr[i], "(-)")){
            strcpy(label,"pdbx_F_minus");
        }
        
    }else if (nset==nn && type_ptr[i][0] == 'L' ) {
        if (strstr(label_ptr[i], "(+)")){
            strcpy(label,"pdbx_F_plus_sigma");
        }else if (strstr(label_ptr[i], "(-)")){
            strcpy(label,"pdbx_F_minus_sigma");
        }

    }else if (nset==nn && type_ptr[i][0] == 'K' ) { //I+/I-
        if (strstr(label_ptr[i], "(+)")){
            strcpy(label,"pdbx_I_plus");
        }else if (strstr(label_ptr[i], "(-)")){
            strcpy(label,"pdbx_I_minus");
        }
        
    }else if (nset==nn && type_ptr[i][0] == 'M' ) {  //sigI+/I-
        if (strstr(label_ptr[i], "(+)")){
            strcpy(label,"pdbx_I_plus_sigma");
        }else if (strstr(label_ptr[i], "(-)")){
            strcpy(label,"pdbx_I_minus_sigma");
        }


    }else if (type_ptr[i][0] == 'I' &&
              (strstr_case(label_ptr[i], "free") ||
               strstr_case(label_ptr[i], "flag") ||
               strstr_case(label_ptr[i], "TEST"))){  //free
        
        strcpy(label,"status");
        

    }else if (nset==nn && type_ptr[i][0] == 'D'){
        strcpy(label,"pdbx_anom_difference");
        

    }else if (nset==nn && type_ptr[i][0] == 'A' ){  //for HL
        if (strstr_case(label_ptr[i], "HLA")){
            strcpy(label,"pdbx_HL_A_iso");
        
        }else if (strstr_case(label_ptr[i], "HLB")){
            strcpy(label,"pdbx_HL_B_iso");
        
        }else if (strstr_case(label_ptr[i], "HLC")){
            strcpy(label,"pdbx_HL_C_iso");
        
        }else if (strstr_case(label_ptr[i], "HLD")){
            strcpy(label,"pdbx_HL_D_iso");
        }

        
    }else if (nset==nn && type_ptr[i][0] == 'P' ){  //phase
        if (!strcmp_case(label_ptr[i], "PHIC") ||
            !strcmp_case(label_ptr[i], "AC")){
            strcpy(label,"phase_calc");
            
        }else if (!strcmp_case(label_ptr[i], "PH2FOFCWT") || !strcmp_case(label_ptr[i], "PHWT")){
            strcpy(label,"pdbx_PHWT");
            
        }else if (!strcmp_case(label_ptr[i], "PHFOFCWT") || !strcmp_case(label_ptr[i], "PHDELWT")){
            strcpy(label,"pdbx_DELPHWT");
            
        }else if (strstr_case(label_ptr[i], "PHIB") ||
                  strstr_case(label_ptr[i], "PHIM")){
            strcpy(label,"phase_meas");
        }

    }else if (nset==nn && type_ptr[i][0] == 'W' && strstr_case(label_ptr[i], "FOM")){  //fom
        strcpy(label,"fom");

    }
              

}

void mtzdmp_file(char *inpfile, char *outfile, long nref)
{
        //use the modified one /apps/ccp4-6.1.3/ccp4-6.1.3/bin/mtzdump_lx
    char  command[256] ;
    sprintf(command, "/bin/csh -c \"source %s/include/ccp4.setup; %s/etc/mtzdmp  %s -n %ld >%s \"", CCP4, CCP4, inpfile, nref, outfile);
    system(command);
}


void mtzdmp2mmcif_native(char *filename, char *outfile, char **label_ptr,
                      char **type_ptr,char **sets_ptr,int ntype, int nset) 
// using mtzdmp and cif parser to get the native data set.
{
    
    char **lines=NULL,  outtmp[256], label[80];
    char labtmp[200][80], pdb_id[80], sym[20], **tok;
    int  i,j,n, num, nline, n1=0, n2=0;
    long nref=90000000;
    float cel[6];
    FILE *fw=NULL;

    strcpy(pdb_id, "xxxx");
    printf("Converting mtz->mmcif for native data set=%d\n", nset);
    
    sprintf(outtmp, "sf_convert.mmcif"); //get log file only
    mtzdmp_file(filename, outtmp, nref);
    
    lines=get_lines_from_file(outtmp, &nline);

    if (nline>nref){
        printf("Error! The file is too big to convert. Please increase ntot.\n");
        exit(0);
    }
    
    
    if((fw=fopen(outtmp, "w"))==NULL){
        printf("Error! file (%s) has no written permission in directory (%s)!\n",
               outtmp, PATHTMP);
        exit(0);
    }

    fprintf(fw,"data_tmp\n");

    for (i=0; i<nline-1; i++){
        
        if (strstr(lines[i], "Cell Dimensions :")){
            tok = string_token(lines[i+1], " " , &n1);
            if (n1==6){
                cel[0]=atof(tok[0]);
                cel[1]=atof(tok[1]);
                cel[2]=atof(tok[2]);
                cel[3]=atof(tok[3]);
                cel[4]=atof(tok[4]);
                cel[5]=atof(tok[5]);
            }
        }else if (pattern5(lines[i], "Space group =", "'","'", "number",  ")" )){
            num=get_int(lines[i], "number", ")");
            tok = string_token(lines[i], "'" , &n2);

            if (n2==3){
                strcpy(sym,tok[1]);
                cryst_infor(fw, cel, sym, num);
                break;
                
            }
        }
    }
    
   
    fprintf(fw,"loop_\n");
    for (i=0; i<ntype; i++){
        get_mtzdmp_label(i, label_ptr, type_ptr, sets_ptr, ntype, nset, label);
        strcpy(labtmp[i], label);
        n=0;
        for (j=i-1; j>=0; j-- ){ // check duplicate
            if (i<1) continue;
            if ( !strcmp_case(labtmp[j], label) ){
                n=1;
                printf("duplicating items: %2d;  _refln.%s & _refln.%s\n", j+1, label_ptr[j], label);
                break;
            }
        }
        if (n==0){
            fprintf(fw,"_refln.%s\n", label);
        }else{
            fprintf(fw,"_refln.%s\n", label_ptr[i]);
        }
        
    }
        //for (i=0; i<ntype; i++){printf("%2d  %20s\n",i, labtmp[i]);}
        
    for (i=1; i<nline; i++){
        if (strstr(lines[i-1], "LIST OF REFLECTIONS") && strstr(lines[i], "===========")){
            for (j=i+1; j<nline; j++){
                if (strstr(lines[j], "COLOR") || strstr(lines[j], "SUMMARY_BEGIN")){
                    break;
                }
                fprintf(fw,"%s",lines[j]);
            }
        }
    }
    fclose(fw);
    
    cif2cif_sf_all(outtmp, outfile, pdb_id, 1);
    delete_file(outtmp);

}

void f_sig_pair(char **label_ptr, char **type_ptr,char **sets_ptr, int ntype,
                const char type1, const char type2, const char lab1, int nset,
                const  char *asign_lab1, const char *asign_lab2, FILE *finp)
// get data pairs (F SIGF, I SIGI, ...); ntype is total number of items
    
{
    char ch;
    int j;

    for(j=3; j<ntype-1; j++){
        
        if(atoi(sets_ptr[j])!= nset) continue; //only for this set
        
        if(strstr(asign_lab1, "(+)")){
            
            if(type_ptr[j][0] ==type1  && type_ptr[j+1][0] == type2){ 
                if(strstr(label_ptr[j], "(+)") && strstr_case(label_ptr[j+1], "S")) {
                        
                    fprintf(finp,"%s=%s  ",asign_lab1, label_ptr[j] );
                    fprintf(finp,"%s=%s  ",asign_lab2, label_ptr[j+1] );
                    break;
                }
            }
        }else if(strstr(asign_lab1, "(-)")){
            
            if(type_ptr[j][0] ==type1  && type_ptr[j+1][0] == type2){ 
                if(strstr(label_ptr[j], "(-)") && strstr_case(label_ptr[j+1], "S")) {
                        
                    fprintf(finp,"%s=%s  ",asign_lab1, label_ptr[j] );
                    fprintf(finp,"%s=%s  ",asign_lab2, label_ptr[j+1] );
                    break;
                }
            }
        }else{

            ch=label_ptr[j][0];
            
//            if(type_ptr[j][0] == type1 && type_ptr[j+1][0] == type2 &&
//               toupper(ch)== lab1 && strstr_case(label_ptr[j+1], "S")){ 

            if(type_ptr[j][0] == type1 && type_ptr[j+1][0] == type2 &&
               (strchr(label_ptr[j], lab1) || toupper(ch)== lab1)
               && strstr_case(label_ptr[j+1], "S") ){ 
                
                fprintf(finp,"%s=%s  ",asign_lab1, label_ptr[j] );
                fprintf(finp,"%s=%s  ",asign_lab2, label_ptr[j+1] );
                break;
                
            }else if(j<ntype-2 &&
                     type_ptr[j][0] == type1 && type_ptr[j+2][0] == type2 &&
                     toupper(ch)== lab1 && strstr_case(label_ptr[j+2], "S")){ 

                
                fprintf(finp,"%s=%s  ",asign_lab1, label_ptr[j] );
                fprintf(finp,"%s=%s  ",asign_lab2, label_ptr[j+2] );
                break;
                
            }else if(type_ptr[j][0] == type1 && type_ptr[j-1][0] == type2 && 
                     toupper(ch)== lab1 && strstr_case(label_ptr[j-1], "S")){ 
                     
                fprintf(finp,"%s=%s  ",asign_lab1, label_ptr[j] );
                fprintf(finp,"%s=%s  ",asign_lab2, label_ptr[j-1] );
                break;
            }
            
        }
        
    }
}

void mtz2mmcif_phs(char *filename, char *outfile, char **label_ptr,
                   char **type_ptr, char **sets_ptr, int ntype, int nset)
// using Peter's script to convert multi-data of MTZ to a single mmCIF file
        
{
    char script[512], command[256];
    int n;
    FILE *fp=NULL;

//    pid=getpid();
    
    sprintf(script, "%smtz2cif_phs_%d.csh",PATHTMP, nset);
    if((fp=fopen(script, "w"))==NULL){
        printf("Error, file (%s) has no written permission in directory (%s)!\n",
               script, PATHTMP);
        exit(0);
    }
    
    fprintf(fp,"#!/bin/csh \n");
    fprintf(fp,"#Script to run the MTZ2CIF program\n");
    fprintf(fp,"#\n");
    fprintf(fp,"#NB MODE keyword allows the user to specify whether to output\n");
    fprintf(fp,"#the standard CCP4 CIF tokens or the RCSB tokens\n\n");

    fprintf(fp,"%s/bin/mtz2cif HKLIN %s HKLOUT %s << EOF> /dev/null\n",
            CCP4, filename,outfile);

    add_labels(fp, label_ptr, type_ptr, sets_ptr, ntype, nset, 2);
   
    fprintf(fp,"datablock data_PHASE_%d\n", nset);
    fprintf(fp,"mode PDBX\n");
    fprintf(fp,"end\n");
    fprintf(fp,"EOF\n\n");

    fclose(fp);

    n=check_mtz_label(script);
    if(n==0) {
        printf("Error! No labels are written for MTZ to mmCIF conversion\n");
        return;
    }
    
    sprintf(command, "/bin/csh -c \"source %s/include/ccp4.setup;chmod +x %s ; %s\" >/dev/null",CCP4,script, script);
    system(command);
//     printf("command=%s\n", command);
   
//    delete_file(script);


}
                         
void mtz2mmcif(char *inpfile, char *outfile)
// get multiple data set.
{
    char **label_ptr=NULL, **type_ptr=NULL, **sets_ptr=NULL;
    char outfile1[256], command[512],label[2000], type[256], sets[256];
    int nlabel=0, ntype=0,nsets=0, i,j,min, max,n_1st,n=0, nfr=0;
    
    get_type_label_sets(inpfile, label, type, sets);
    
    label_ptr = string_token(label," ", &nlabel);
    type_ptr = string_token(type," ", &ntype);
    sets_ptr = string_token(sets," ", &nsets);

    if(nlabel != ntype || nlabel != nsets ||  nsets<=3){
        printf("Error! There is a problem with data set.\n");
        return;
    }
    min=90;
    max=-1;
    
    printf("items  set  type  label\n");
    for(i=0; i<nsets; i++){
        printf("   %2d   %s    %s     %s\n",
               i+1,sets_ptr[i], type_ptr[i], label_ptr[i]);
        if (type_ptr[i][0]=='I' && (strstr(label_ptr[i], "R-free-flags(+)") ||
                                    strstr(label_ptr[i], "R-free-flags(-)")))nfr++;
    }
    
    for(i=3; i<nsets; i++){
        j=atoi(sets_ptr[i]);
        if(j<min)min=j;
        if(j>max)max=j;
    }

    n_1st=first_set_num(label_ptr, type_ptr, sets_ptr, ntype);
    
    printf("Total Columns=%d, min=%d, max=%d, native data set=%d\n",ntype, min,max, n_1st);
    
    if(ADD_MAP==0 && nfr !=2 ){
        mtz2mmcif_native(inpfile, outfile, label_ptr, type_ptr, sets_ptr, ntype, n_1st); 
    }else{
        if (nfr==2){
            printf("Note: there are two flags for the free sets\n");
        }
        
        mtzdmp2mmcif_native(inpfile, outfile, label_ptr, type_ptr, sets_ptr, ntype, n_1st);
    }
    
    for (i=min; i<=max; i++){
        if (i ==0 || i==n_1st) continue;
        n++;
        sprintf(outfile1, "%s.out_%d", inpfile,n);
        printf("Converting mtz->mmcif for phase data set=%d\n", i);
        
        mtz2mmcif_phs(inpfile, outfile1, label_ptr, type_ptr, sets_ptr, ntype, i);
        sprintf(command, "/bin/csh -c \" /bin/cat  %s >> %s\" ",outfile1,outfile);
        system(command);
        delete_file(outfile1);
    }
}
